import os
import toml

def get_version_from_pyproject():
    # List of potential locations for the pyproject.toml file
    potential_paths = [
        os.path.join(os.path.dirname(__file__), '../../pyproject.toml'),
        os.path.join(os.path.dirname(__file__), '../pyproject.toml'),
        os.path.join(os.path.dirname(__file__), 'pyproject.toml'),
    ]

    for path in potential_paths:
        if os.path.exists(path):
            with open(path, 'r') as f:
                return toml.loads(f.read())['tool']['poetry']['version']

    # Raise an error if the file is not found in any of the locations
    raise FileNotFoundError("pyproject.toml not found in expected locations")



def convert_to_openapi_example(configuration_examples):
    return{k: {"summary": "{example}".format(example=k),"value":v} for k, v in configuration_examples.items()}


server_configuration_examples = {
    "emptyserver": {},
    "DellR740": {"model":
            {
                "type": "rack"
            },
        "configuration":
            {
                "cpu":
                    {
                        "units": 2,
                        "name": "intel xeon gold 6134"
                    },
                "ram":
                    [
                        {
                            "units": 12,
                            "capacity": 32

                        }
                    ],
                "disk":
                    [
                        {
                            "units": 1,
                            "type": "ssd",
                            "capacity": 400,
                            "density": 50.6
                        }
                    ]
            },
        "usage": {
            "avg_power": 300,
            "usage_location": "FRA"
        }
    }}

server_configuration_examples_openapi = convert_to_openapi_example(server_configuration_examples)

components_examples = {
    "cpu": {
        "intel xeon gold 6134":{
            "name": "intel xeon gold 6134",
        },
        "empty cpu":{},
        "custom cpu":{
            "core_units":64,
            "tdp": 150,
            "die_size": 400,
            "usage":{
                "usage_location":"FRA"
            }
        }
    },
    "ssd": {
        "custom ssd" :{
            "capacity": 500,
            "manufacturer": "Samsung",
        },
        "empty ssd":{}
    },
    "ram": {
        "custom ram":{
            "capacity": 32,
            "manufacturer": "Samsung",
            "process": 30.0
        },
        "empty ram":{}
    },
    "hdd": {
        "empty HDD":{}
    },
    "motherboard": {
        "empty motherboard":{}
    },
    "power_supply": {
        "custom power supply":{
            "unit_weight": 10
        }
    },
    "case": {
        "rack":{
            "case_type": "rack"
        },
        "blade":{
            "case_type": "blade"
        }
    }
}

components_examples_openapi = {
    "cpu": convert_to_openapi_example(components_examples["cpu"]),
    "ram": convert_to_openapi_example(components_examples["ram"]),
    "ssd": convert_to_openapi_example(components_examples["ssd"]),
    "hdd": convert_to_openapi_example(components_examples["hdd"]),
    "motherboard": convert_to_openapi_example(components_examples["motherboard"]),
    "power_supply": convert_to_openapi_example(components_examples["power_supply"]),
    "case": convert_to_openapi_example(components_examples["case"]),
}

cloud_usage_example = {
        "usage_location": "FRA",
        "time_workload": [
            {"time_percentage": 50, "load_percentage": 0},
            {"time_percentage": 25, "load_percentage": 60},
            {"time_percentage": 25, "load_percentage": 100}
        ]
}

cloud_example = {
    "provider": "aws",
    "instance_type": "a1.4xlarge",
    "usage":
            {
            "usage_location": "FRA",
            "time_workload": [
                {"time_percentage": 50, "load_percentage": 0},
                {"time_percentage": 25, "load_percentage": 60},
                {"time_percentage": 25, "load_percentage": 100}
            ]}
}

cpu_consumption_profiles = {
        "cpu": {
            "name": "intel xeon gold 6134"
        },
        "workload": [
            {
                "load_percentage": 0,
                "power_watt": 50
            },
            {
                "load_percentage": 10,
                "power_watt": 100
            },{
                "load_percentage": 50,
                "power_watt": 180
            },
            {
                "load_percentage": 100,
                "power_watt": 245
            }
        ],
}

end_user_terminal = {
    "usage": {
        "use_time_ratio": 0.3,
        "usage_location": "FRA",
    }
}


from typing import Optional


from boaviztapi import config
from boaviztapi.dto.component import ComponentDTO
from boaviztapi.dto.usage.usage import mapper_usage, Usage
from boaviztapi.model.component import ComponentSSD, ComponentHDD
from boaviztapi.service.archetype import get_component_archetype


class Disk(ComponentDTO):
    type: Optional[str] = None
    capacity: Optional[int] = None
    density: Optional[float] = None
    manufacturer: Optional[str] = None
    model: Optional[str] = None
    layers: Optional[int] = None


def mapper_ssd(disk_dto: Disk, archetype=get_component_archetype(config["default_ssd"], "ssd")) -> ComponentSSD:
    disk_component = ComponentSSD(archetype=archetype)
    disk_component.usage = mapper_usage(disk_dto.usage or Usage(), archetype=archetype.get("USAGE"))

    if disk_dto.units is not None:
        disk_component.units.set_input(disk_dto.units)

    if disk_dto.capacity is not None:
        disk_component.capacity.set_input(disk_dto.capacity)

    if disk_dto.manufacturer is not None:
        disk_component.manufacturer.set_input(disk_dto.manufacturer)

    if disk_dto.density is not None:
        disk_component.density.set_input(disk_dto.density)

    if disk_dto.layers is not None:
        disk_component.layers.set_input(disk_dto.layers)

    return disk_component


def mapper_hdd(disk_dto: Disk, archetype=get_component_archetype(config["default_hdd"], "hdd")) -> ComponentHDD:
    disk_component = ComponentHDD(archetype=archetype)
    disk_component.usage = mapper_usage(disk_dto.usage or Usage(), archetype=archetype.get("USAGE"))

    if disk_dto.units is not None:
        disk_component.units.set_input(disk_dto.units)
    if disk_dto.capacity is not None:
        disk_component.capacity.set_input(disk_dto.capacity)

    return disk_component



import os
from typing import List, Optional

from fastapi import APIRouter, Body, HTTPException, Query

from boaviztapi import config, data_dir
from boaviztapi.dto.component import CPU, RAM, Disk, PowerSupply, Motherboard, Case
from boaviztapi.dto.component.cpu import mapper_cpu
from boaviztapi.dto.component.other import mapper_motherboard, mapper_power_supply, mapper_case
from boaviztapi.dto.component.ram import mapper_ram
from boaviztapi.dto.component.disk import mapper_ssd, mapper_hdd
from boaviztapi.model.component import Component
from boaviztapi.routers.openapi_doc.descriptions import cpu_description, ram_description, ssd_description, \
    hdd_description, motherboard_description, power_supply_description, case_description
from boaviztapi.routers.openapi_doc.examples import components_examples_openapi
from boaviztapi.service.archetype import get_component_archetype, get_device_archetype_lst
from boaviztapi.service.impacts_computation import compute_impacts
from boaviztapi.service.verbose import verbose_component

component_router = APIRouter(
    prefix='/v1/component',
    tags=['component']
)


@component_router.get('/all',
                      description=cpu_description)
async def cpu_all_archetype_name():
    return {
        "cpu": "v1/component/cpu",
        "ram": "v1/component/ram",
        "ssd": "v1/component/ssd",
        "hdd": "v1/component/hdd",
        "motherboard": "v1/component/motherboard",
        "power_supply": "v1/component/power_supply",
        "case": "v1/component/case"
    }


@component_router.get('/cpu/archetype',
                      description=cpu_description)
async def cpu_all_archetype_name():
    archetype_lst = get_all_archetype_name("cpu")
    return archetype_lst


@component_router.get('/cpu/archetype_config',
                      description=cpu_description)
async def cpu_archetype_config(archetype: str = Query(example=config["default_cpu"])):
    archetype_config = get_archetype_config(archetype, "cpu")
    return archetype_config


@component_router.post('/cpu',
                       description=cpu_description)
async def cpu_impact_bottom_up(cpu: CPU = Body(None, openapi_examples=components_examples_openapi["cpu"]),
                               verbose: bool = True,
                               duration: Optional[float] = config["default_duration"],
                               archetype: str = config["default_cpu"],
                               criteria: List[str] = Query(config["default_criteria"])):
    archetype_config = get_component_archetype(archetype, "cpu")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    component = mapper_cpu(cpu, archetype_config)

    return await component_impact_bottom_up(
        component=component,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/cpu',
                      description=cpu_description)
async def cpu_impact_bottom_up(verbose: bool = True,
                               duration: Optional[float] = config["default_duration"],
                               archetype: str = config["default_cpu"],
                               criteria: List[str] = Query(config["default_criteria"])):
    archetype_config = get_component_archetype(archetype, "cpu")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    component = mapper_cpu(CPU(), archetype_config)

    return await component_impact_bottom_up(
        component=component,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/ram/archetype',
                      description=ram_description)
async def ram_all_archetype_name():
    archetype_lst = get_all_archetype_name("ram")
    return archetype_lst


@component_router.get('/ram/archetype_config',
                      description=ram_description)
async def ram_archetype_config(archetype: str = Query(example=config["default_ram"])):
    archetype_config = get_archetype_config(archetype, "ram")
    return archetype_config


@component_router.post('/ram',
                       description=ram_description)
async def ram_impact_bottom_up(ram: RAM = Body(None, openapi_examples=components_examples_openapi["ram"]),
                               verbose: bool = True,
                               duration: Optional[float] = config["default_duration"],
                               archetype: str = config["default_ram"],
                               criteria: List[str] = Query(config["default_criteria"])):
    archetype_config = get_component_archetype(archetype, "ram")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    component = mapper_ram(ram, archetype_config)

    return await component_impact_bottom_up(
        component=component,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/ram',
                      description=ram_description)
async def ram_impact_bottom_up(verbose: bool = True,
                               duration: Optional[float] = config["default_duration"],
                               archetype: str = config["default_ram"],
                               criteria: List[str] = Query(config["default_criteria"])):
    archetype_config = get_component_archetype(archetype, "ram")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    component = mapper_ram(RAM(), archetype_config)

    return await component_impact_bottom_up(
        component=component,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/ssd/archetype',
                      description=ssd_description)
async def ssd_all_archetype_name():
    archetype_lst = get_all_archetype_name("ssd")
    return archetype_lst


@component_router.get('/ssd/archetype_config',
                      description=ssd_description)
async def ssd_archetype_config(archetype: str = Query(example=config["default_ssd"])):
    archetype_config = get_archetype_config(archetype, "ssd")
    return archetype_config


@component_router.post('/ssd',
                       description=ssd_description)
async def disk_impact_bottom_up(disk: Disk = Body(None, openapi_examples=components_examples_openapi["ssd"]),
                                verbose: bool = True,
                                duration: Optional[float] = config["default_duration"],
                                archetype: str = config["default_ssd"],
                                criteria: List[str] = Query(config["default_criteria"])):
    disk.type = "ssd"
    archetype_config = get_component_archetype(archetype, "ssd")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    component = mapper_ssd(disk, archetype_config)

    return await component_impact_bottom_up(
        component=component,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/ssd',
                      description=ssd_description)
async def disk_impact_bottom_up(verbose: bool = True,
                                duration: Optional[float] = config["default_duration"],
                                archetype: str = config["default_ssd"],
                                criteria: List[str] = Query(config["default_criteria"])):
    disk = Disk()
    disk.type = "ssd"
    archetype_config = get_component_archetype(archetype, "ssd")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    component = mapper_ssd(disk, archetype_config)

    return await component_impact_bottom_up(
        component=component,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/hdd/archetype',
                      description=hdd_description)
async def hdd_all_archetype_name():
    archetype_lst = get_all_archetype_name("hdd")
    return archetype_lst


@component_router.get('/hdd/archetype_config',
                      description=hdd_description)
async def hdd_archetype_config(archetype: str = Query(example=config["default_hdd"])):
    archetype_config = get_archetype_config(archetype, "hdd")
    return archetype_config


@component_router.post('/hdd',
                       description=hdd_description)
async def disk_impact_bottom_up(disk: Disk = Body(None, openapi_examples=components_examples_openapi["hdd"]),
                                verbose: bool = True,
                                duration: Optional[float] = config["default_duration"],
                                archetype: str = config["default_hdd"],
                                criteria: List[str] = Query(config["default_criteria"])):
    disk.type = "hdd"
    archetype_config = get_component_archetype(archetype, "hdd")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    component = mapper_hdd(disk, archetype_config)

    return await component_impact_bottom_up(
        component=component,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/hdd',
                      description=hdd_description)
async def disk_impact_bottom_up(verbose: bool = True,
                                duration: Optional[float] = config["default_duration"],
                                archetype: str = config["default_hdd"],
                                criteria: List[str] = Query(config["default_criteria"])):
    disk = Disk()
    disk.type = "hdd"
    archetype_config = get_component_archetype(archetype, "hdd")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    component = mapper_hdd(disk, archetype_config)

    return await component_impact_bottom_up(
        component=component,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/motherboard/archetype',
                      description=motherboard_description)
async def motherboard_all_archetype_name():
    archetype_lst = get_all_archetype_name("motherboard")
    return archetype_lst


@component_router.get('/motherboard/archetype_config',
                      description=motherboard_description)
async def motherboard_archetype_config(archetype: str = Query(example=config["default_motherboard"])):
    archetype_config = get_archetype_config(archetype, "motherboard")
    return archetype_config


@component_router.post('/motherboard',
                       description=motherboard_description)
async def motherboard_impact_bottom_up(
        motherboard: Motherboard = Body(None, openapi_examples=components_examples_openapi["motherboard"]),
        verbose: bool = True,
        duration: Optional[float] = config["default_duration"],
        criteria: List[str] = Query(config["default_criteria"])):
    completed_motherboard = mapper_motherboard(motherboard)

    return await component_impact_bottom_up(
        component=completed_motherboard,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/motherboard',
                      description=motherboard_description)
async def motherboard_impact_bottom_up(verbose: bool = True,
                                       duration: Optional[float] = config["default_duration"],
                                       criteria: List[str] = Query(config["default_criteria"])):
    completed_motherboard = mapper_motherboard(Motherboard())

    return await component_impact_bottom_up(
        component=completed_motherboard,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/power_supply/archetype',
                      description=power_supply_description)
async def power_supply_all_archetype_name():
    archetype_lst = get_all_archetype_name("power_supply")
    return archetype_lst


@component_router.get('/power_supply/archetype_config',
                      description=power_supply_description)
async def power_supply_archetype_config(archetype: str = Query(example=config["default_power_supply"])):
    archetype_config = get_archetype_config(archetype, "power_supply")
    return archetype_config


@component_router.post('/power_supply',
                       description=power_supply_description)
async def power_supply_impact_bottom_up(
        power_supply: PowerSupply = Body(None, openapi_examples=components_examples_openapi["power_supply"]),
        verbose: bool = True,
        duration: Optional[float] = config["default_duration"],
        archetype: str = config["default_power_supply"],
        criteria: List[str] = Query(config["default_criteria"])):
    archetype_config = get_component_archetype(archetype, "power_supply")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    completed_power_supply = mapper_power_supply(power_supply, archetype_config)

    return await component_impact_bottom_up(
        component=completed_power_supply,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/power_supply',
                      description=power_supply_description)
async def power_supply_impact_bottom_up(verbose: bool = True,
                                        duration: Optional[float] = config["default_duration"],
                                        archetype: str = config["default_power_supply"],
                                        criteria: List[str] = Query(config["default_criteria"])):
    archetype_config = get_component_archetype(archetype, "power_supply")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    completed_power_supply = mapper_power_supply(PowerSupply(), archetype_config)

    return await component_impact_bottom_up(
        component=completed_power_supply,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/case/archetype',
                      description=case_description)
async def case_all_archetype_name():
    archetype_lst = get_all_archetype_name("case")
    return archetype_lst


@component_router.get('/case/archetype_config',
                      description=case_description)
async def case_archetype_config(archetype: str = Query(example=config["default_case"])):
    archetype_config = get_archetype_config(archetype, "case")
    return archetype_config


@component_router.post('/case',
                       description=case_description)
async def case_impact_bottom_up(case: Case = Body(None, openapi_examples=components_examples_openapi["case"]),
                                verbose: bool = True,
                                duration: Optional[float] = config["default_duration"],
                                archetype: str = config["default_case"],
                                criteria: List[str] = Query(config["default_criteria"])):
    archetype_config = get_component_archetype(archetype, "case")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    completed_case = mapper_case(case, archetype_config)

    return await component_impact_bottom_up(
        component=completed_case,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@component_router.get('/case',
                      description=case_description)
async def case_impact_bottom_up(verbose: bool = True,
                                duration: Optional[float] = config["default_duration"],
                                archetype: str = config["default_case"],
                                criteria: List[str] = Query(config["default_criteria"])):
    archetype_config = get_component_archetype(archetype, "case")

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    completed_case = mapper_case(Case(), archetype_config)

    return await component_impact_bottom_up(
        component=completed_case,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


async def component_impact_bottom_up(component: Component,
                                     verbose: bool,
                                     duration: Optional[float] = config["default_duration"],
                                     criteria=config["default_criteria"]) -> dict:
    if duration is None:
        duration = component.usage.hours_life_time.value

    impacts = compute_impacts(model=component, duration=duration, selected_criteria=criteria)

    if verbose:
        return {
            "impacts": impacts,
            "verbose": verbose_component(component=component, duration=duration)
        }
    return {"impacts": impacts}


def get_all_archetype_name(name: str):
    return get_device_archetype_lst(os.path.join(data_dir, f'archetypes/components/{name.lower()}.csv'))


def get_archetype_config(archetype: str, component_type: str):
    result = get_component_archetype(archetype, component_type)
    if not result:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")
    return result



from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component import Component
from boaviztapi.service.archetype import get_arch_value


class ComponentFunctionalBlock(Component):
    NAME = "FUNCTIONAL_BLOCK"
    IMPACT_KEY = None

    def __init__(self, archetype=None, **kwargs):
        super().__init__(archetype=archetype, **kwargs)
        self.hsl_level = Boattribute(
            unit="none",
            default=get_arch_value(archetype, 'hsl_level', 'default'),
            min=get_arch_value(archetype, 'hsl_level', 'min'),
            max=get_arch_value(archetype, 'hsl_level', 'max')
        )


class ActuatorsFunctionalBlock(ComponentFunctionalBlock):
    NAME = "ACTUATORS"
    IMPACT_KEY = "actuators"


class CasingFunctionalBlock(ComponentFunctionalBlock):
    NAME = "CASING"
    IMPACT_KEY = "casing"


class ConnectivityFunctionalBlock(ComponentFunctionalBlock):
    NAME = "CONNECTIVITY"
    IMPACT_KEY = "connectivity"


class MemoryFunctionalBlock(ComponentFunctionalBlock):
    NAME = "MEMORY"
    IMPACT_KEY = "memory"


class OthersFunctionalBlock(ComponentFunctionalBlock):
    NAME = "OTHERS"
    IMPACT_KEY = "others"


class PcbFunctionalBlock(ComponentFunctionalBlock):
    NAME = "PCB"
    IMPACT_KEY = "pcb"


class PowerSupplyFunctionalBlock(ComponentFunctionalBlock):
    NAME = "POWER_SUPPLY"
    IMPACT_KEY = "power_supply"


class SecuritySupplyFunctionalBlock(ComponentFunctionalBlock):
    NAME = "SECURITY"
    IMPACT_KEY = "security"


class ProcessingSupplyFunctionalBlock(ComponentFunctionalBlock):
    NAME = "PROCESSING"
    IMPACT_KEY = "processing"


class SensingSupplyFunctionalBlock(ComponentFunctionalBlock):
    NAME = "SENSING"
    IMPACT_KEY = "sensing"


class UserInterfaceSupplyFunctionalBlock(ComponentFunctionalBlock):
    NAME = "USER_INTERFACE"
    IMPACT_KEY = "user_interface"


def get_functional_block(name: str):
    if name == ActuatorsFunctionalBlock.NAME:
        return ActuatorsFunctionalBlock
    elif name == CasingFunctionalBlock.NAME:
        return CasingFunctionalBlock
    elif name == ConnectivityFunctionalBlock.NAME:
        return ConnectivityFunctionalBlock
    elif name == MemoryFunctionalBlock.NAME:
        return MemoryFunctionalBlock
    elif name == OthersFunctionalBlock.NAME:
        return OthersFunctionalBlock
    elif name == PcbFunctionalBlock.NAME:
        return PcbFunctionalBlock
    elif name == PowerSupplyFunctionalBlock.NAME:
        return PowerSupplyFunctionalBlock
    elif name == SecuritySupplyFunctionalBlock.NAME:
        return SecuritySupplyFunctionalBlock
    elif name == ProcessingSupplyFunctionalBlock.NAME:
        return ProcessingSupplyFunctionalBlock
    elif name == SensingSupplyFunctionalBlock.NAME:
        return SensingSupplyFunctionalBlock
    elif name == UserInterfaceSupplyFunctionalBlock.NAME:
        return UserInterfaceSupplyFunctionalBlock
    else:
        raise ValueError("Unknown functional block name: {}".format(name))



import ast
import csv
import os
from typing import Union

import pandas as pd

from boaviztapi import data_dir


def get_device_archetype_lst(path):
    df = pd.read_csv(path)
    return df['id'].tolist()


def get_device_archetype_lst_with_type(path, name: str, ) -> Union[dict, bool]:
    df = pd.read_csv(path)
    df = df[df['device_type'] == name]
    return df['id'].tolist()


def get_component_archetype(archetype_name: str, component_type: str) -> Union[dict, bool]:
    arch = get_archetype(archetype_name, os.path.join(data_dir, "archetypes/components/" + component_type + ".csv"))
    if not arch:
        return False
    return arch


def get_server_archetype(archetype_name: str) -> Union[dict, bool]:
    arch = get_archetype(archetype_name, os.path.join(data_dir, "archetypes/server.csv"))
    if not arch:
        return False
    return arch


def get_user_terminal_archetype(archetype_name: str) -> Union[dict, bool]:
    arch = get_archetype(archetype_name, os.path.join(data_dir, "archetypes/user_terminal.csv"))
    if not arch:
        return False
    return arch


def get_cloud_instance_archetype(archetype_name: str, provider: str) -> Union[dict, bool]:
    arch = False
    if os.path.exists(data_dir + "/archetypes/cloud/" + provider + ".csv"):
        arch = get_archetype(archetype_name, os.path.join(data_dir, "archetypes/cloud/" + provider + ".csv"))
    if not arch:
        return False
    return arch


def get_archetype(archetype_name: str, csv_path: str) -> Union[dict, bool]:
    with open(csv_path, encoding='utf-8') as csvfile:
        reader = csv.DictReader(csvfile)
        for row in reader:
            if row["id"].strip() == archetype_name.strip():
                return row2json(row)
    return False


def parse_to_boattribute_json(value):
    json = {}
    if value == "" or value is None:
        return json
    elif ";" in value:
        values = value.split(";")
        if len(values) == 3:
            json["default"] = convert(values[0])
            json["min"] = convert(values[1])
            json["max"] = convert(values[2])
        if len(values) == 2:
            json["default"] = convert(values[0])
            if convert(values[1]) > convert(values[0]):
                json["min"] = convert(values[0])
                json["max"] = convert(values[1])
            else:
                json["min"] = convert(values[1])
                json["max"] = convert(values[0])
    else:
        json["default"] = convert(value)
    return json


def row2json(archetype):
    obj = {}
    for attribute in archetype:
        if attribute == "id":
            continue
        value = parse_to_boattribute_json(archetype[attribute])
        names = attribute.split('.')
        nested_set(obj, names, value)
    obj = set_list(obj)
    return obj


def nested_set(dic, keys, value):
    for key in keys[:-1]:
        dic = dic.setdefault(key, {})
    dic[keys[-1]] = value


def set_list(obj):
    if obj.get("configuration") is not None:
        if obj.get("configuration").get("disk"):
            obj["configuration"]["disk"] = [obj["configuration"]["disk"]]
        if obj.get("configuration").get("ram"):
            obj["configuration"]["ram"] = [obj["configuration"]["ram"]]
    return obj


def get_arch_value(archetype: dict, attribute: str, key: str, default=None):
    if not archetype:
        return default
    if archetype.get(attribute) is not None:
        if archetype.get(attribute).get(key) is not None:
            return archetype.get(attribute).get(key)
    return default


def get_arch_component(archetype: dict, component_name: str, default=None):
    if not archetype:
        return default
    if archetype.get(component_name) is not None:
        if component_name != "USAGE" and archetype.get("USAGE") is not None:
            archetype[component_name]["USAGE"] = archetype.get("USAGE")
        return archetype.get(component_name)
    return default


def get_iot_device_archetype(archetype_name: str) -> Union[dict, bool]:
    arch = get_archetype(archetype_name, os.path.join(data_dir, "archetypes/iot_device.csv"))
    if not arch:
        return False
    return arch

def convert(value):
    try:
        value_float = float(value)
        return value_float
    except ValueError:
        if "{" in value and "}" in value or "[" in value and "]" in value:
            try:
                value_dict = ast.literal_eval(value)
                if isinstance(value_dict, dict):
                    return value_dict
            except ValueError:
                pass

    return value


from typing import Tuple, Union, Optional

from boaviztapi import config
from boaviztapi.dto.component import Motherboard
from boaviztapi.model import ComputedImpacts
from boaviztapi.model.device.server import DeviceServer
from boaviztapi.model.services.cloud_instance import Service, ServiceCloudInstance
from boaviztapi.model.component import ComponentCPU, ComponentCase, ComponentPowerSupply, ComponentRAM, Component, \
    ComponentAssembly, ComponentHDD, ComponentSSD, ComponentMotherboard
from boaviztapi.model.component.functional_block import ComponentFunctionalBlock
from boaviztapi.model.device import Device
from boaviztapi.model.device.iot import DeviceIoT
from boaviztapi.model.impact import ImpactFactor, IMPACT_PHASES, IMPACT_CRITERIAS, Impact, USE
from boaviztapi.service.factor_provider import get_impact_factor, get_iot_impact_factor


def compute_single_impact(model: Union[Component, Device, Service],
                          phase: str,
                          criteria: str,
                          duration: Union[int, str] = config["default_duration"],
                          allocation: float = 1) -> Optional[Impact]:
    try:
        impact_function = get_impact_function(model, phase)

        impact, min_impact, max_impact, warnings = impact_function(criteria, duration, model)

        result = Impact(
            value=impact * allocation,
            min=min_impact * allocation,
            max=max_impact * allocation,
            warnings=list(set(warnings))
        )

        model.add_impacts(result, criteria, phase)

        return result
    except (AttributeError, NotImplementedError):
        model.add_impacts(None, criteria, phase)
        return None


def compute_impacts(model: Union[Component, Device, Service], selected_criteria=config["default_criteria"],
                    duration=config["default_duration"]) -> dict:
    for c in IMPACT_CRITERIAS.keys():
        criteria = IMPACT_CRITERIAS[c]
        if "all" not in selected_criteria:
            if criteria.name not in selected_criteria:
                continue
        for phase in IMPACT_PHASES:
            compute_single_impact(model, phase, criteria.name, duration)

    return model.get_impacts(selected_criteria)


def get_impact_function(model: Union[Component, Device, Service], phase: str):
    return impacts_functions[model.NAME][phase]


def not_implemented_function(impact_type: str, duration: int, model: Union[Component, Device, Service]):
    raise NotImplementedError


def simple_impact_use(impact_type: str, duration: int, model: Union[Component, Device, Service]) -> ComputedImpacts:
    if not model.usage.avg_power.has_value():
        raise NotImplementedError

    impact_factor = model.usage.elec_factors[impact_type]

    impacts = impact_factor.value * (model.usage.avg_power.value / 1000) * model.usage.use_time_ratio.value * duration * model.units.value
    max_impact = impact_factor.max * (model.usage.avg_power.max / 1000) * model.usage.use_time_ratio.min * duration * model.units.max
    min_impact = impact_factor.min * (model.usage.avg_power.min / 1000) * model.usage.use_time_ratio.max * duration * model.units.min

    return impacts, min_impact, max_impact, []


def simple_embedded(impact_type: str, duration: int, model: [Device, Component, Service]) -> ComputedImpacts:
    if hasattr(model, 'type') and model.type is not None:
        impact = Impact(
            value=float(get_impact_factor(item=model.NAME, impact_type=impact_type)[model.type.value]["impact"]) * model.units.value,
            min=float(get_impact_factor(item=model.NAME, impact_type=impact_type)[model.type.value]["impact"]) * model.units.min,
            max=float(get_impact_factor(item=model.NAME, impact_type=impact_type)[model.type.value]["impact"]) * model.units.max)
    else:
        impact = Impact(
            value=float(get_impact_factor(item=model.NAME, impact_type=impact_type)["impact"]) * model.units.value,
            min=float(get_impact_factor(item=model.NAME, impact_type=impact_type)["impact"]) * model.units.min,
            max=float(get_impact_factor(item=model.NAME, impact_type=impact_type)["impact"]) * model.units.max)

    impact.allocate(duration, model.usage.hours_life_time)

    warnings = ["Generic data used for impact calculation."]

    return impact.value, impact.min, impact.max, warnings


def cpu_impact_use(impact_type: str, duration: int, cpu: ComponentCPU) -> ComputedImpacts:
    impact_factor = cpu.usage.elec_factors[impact_type]

    if not cpu.usage.avg_power.is_set():
        modeled_consumption = cpu.model_power_consumption()
        cpu.usage.avg_power.set_completed(
            modeled_consumption.value,
            min=modeled_consumption.min,
            max=modeled_consumption.max
        )

    impact = Impact(
        value=impact_factor.value * (
                cpu.usage.avg_power.value / 1000) * cpu.usage.use_time_ratio.value * duration,
        min=impact_factor.min * (
                cpu.usage.avg_power.min / 1000) * cpu.usage.use_time_ratio.min * duration,
        max=impact_factor.max * (
                cpu.usage.avg_power.max / 1000) * cpu.usage.use_time_ratio.max * duration
    )

    return impact.value, impact.min, impact.max, []


def cpu_impact_embedded(impact_type: str, duration: int, cpu: ComponentCPU) -> ComputedImpacts:
    cpu_die_impact = Impact(
        value=get_impact_factor(item='cpu', impact_type=impact_type)['die_impact'],
        min=get_impact_factor(item='cpu', impact_type=impact_type)['die_impact'],
        max=get_impact_factor(item='cpu', impact_type=impact_type)['die_impact']
    )
    cpu_impact = Impact(
        value=get_impact_factor(item='cpu', impact_type=impact_type)['impact'],
        min=get_impact_factor(item='cpu', impact_type=impact_type)['impact'],
        max=get_impact_factor(item='cpu', impact_type=impact_type)['impact']
    )

    impact = Impact(
        value=(cpu.die_size.value * cpu_die_impact.value + cpu_impact.value) * cpu.units.value,
        min=(cpu.die_size.min * cpu_die_impact.min + cpu_impact.min) * cpu.units.min,
        max=(cpu.die_size.max * cpu_die_impact.max + cpu_impact.max ) * cpu.units.max)

    impact.allocate(duration, cpu.usage.hours_life_time)

    return impact.value, impact.min, impact.max, ["End of life is not included in the calculation"]


def assembly_impact_embedded(impact_type: str, duration: int, model: ComponentAssembly) -> ComputedImpacts:
    impact = Impact(
        value=get_impact_factor(item='assembly', impact_type=impact_type)['impact'] * model.units.value,
        min=get_impact_factor(item='assembly', impact_type=impact_type)['impact'] * model.units.min,
        max=get_impact_factor(item='assembly', impact_type=impact_type)['impact'] * model.units.max
    )

    impact.allocate(duration, model.usage.hours_life_time)

    return impact.value, impact.min, impact.max, ["End of life is not included in the calculation"]


def casing_impact_embedded(impact_type: str, duration: int, case: ComponentCase) -> ComputedImpacts:
    if case.case_type.value == 'rack':
        computed_impact = impact_manufacture_rack(impact_type, case)
    elif case.case_type.value == 'blade':
        computed_impact = impact_manufacture_blade(impact_type, case)
    else:
        computed_impact = impact_manufacture_rack(impact_type, case)

    impact = Impact(
        value=computed_impact[0] * case.units.value,
        min=computed_impact[1] * case.units.min,
        max=computed_impact[2] * case.units.max
    )

    impact.allocate(duration, case.usage.hours_life_time)

    return impact.value, impact.min, impact.max, ["End of life is not included in the calculation"]


def impact_manufacture_rack(impact_type: str, case: ComponentCase) -> ComputedImpacts:
    impact_factor = Impact(
        value=get_impact_factor(item='case', impact_type=impact_type)['rack']['impact'],
        min=get_impact_factor(item='case', impact_type=impact_type)['rack']['impact'],
        max=get_impact_factor(item='case', impact_type=impact_type)['rack']['impact']
    )

    if case.case_type.is_archetype() and case.case_type.value == 'rack':
        blade_impact = impact_manufacture_blade(impact_type, case)
        if blade_impact[0] > impact_factor.value:
            return impact_factor.value, impact_factor.min, blade_impact[2], [
                "End of life is not included in the calculation"]
        else:
            return impact_factor.value, blade_impact[1], impact_factor.max, [
                "End of life is not included in the calculation"]
    return impact_factor.value * case.units.value, impact_factor.min * case.units.min, impact_factor.max * case.units.max, ["End of life is not included in the calculation"]


def impact_manufacture_blade(impact_type: str, case: ComponentCase) -> ComputedImpacts:
    impact_blade_server, impact_blade_16_slots = get_impact_constants_blade(impact_type)

    impact = compute_impact_manufacture_blade(impact_blade_server, impact_blade_16_slots)

    if case.case_type.is_archetype() and case.case_type.value == 'blade':
        rack_impact = impact_manufacture_rack(impact_type, case)
        if rack_impact[0] > impact.value:
            return impact.value, impact.min, rack_impact[2], ["End of life is not included in the calculation"]
        else:
            return impact.value, rack_impact[1], impact.max, ["End of life is not included in the calculation"]

    return impact.value * case.units.value, impact.min * case.units.min, impact.max * case.units.max, ["End of life is not included in the calculation"]


def get_impact_constants_blade(impact_type: str) -> Tuple[Impact, Impact]:
    impact_blade_server = Impact(
        value=get_impact_factor(item='case', impact_type=impact_type)['blade']['impact_blade_server'],
        min=get_impact_factor(item='case', impact_type=impact_type)['blade']['impact_blade_server'],
        max=get_impact_factor(item='case', impact_type=impact_type)['blade']['impact_blade_server'],
    )
    impact_blade_16_slots = Impact(
        value=get_impact_factor(item='case', impact_type=impact_type)['blade']['impact_blade_16_slots'],
        min=get_impact_factor(item='case', impact_type=impact_type)['blade']['impact_blade_16_slots'],
        max=get_impact_factor(item='case', impact_type=impact_type)['blade']['impact_blade_16_slots'],
    )

    return impact_blade_server, impact_blade_16_slots


def compute_impact_manufacture_blade(impact_blade_server: Impact,
                                     impact_blade_16_slots: Impact) -> ImpactFactor:
    return ImpactFactor(
        value=(impact_blade_16_slots.value / 16) + impact_blade_server.value,
        min=(impact_blade_16_slots.min / 16) + impact_blade_server.min,
        max=(impact_blade_16_slots.max / 16) + impact_blade_server.max
    )


def iot_functional_blocks_impact_embedded(impact_type: str, duration: int,
                                          function_blocks: ComponentFunctionalBlock) -> ComputedImpacts:
    impact = Impact(
        value=get_iot_impact_factor(function_blocks.IMPACT_KEY, function_blocks.hsl_level.value, impact_type) * function_blocks.units.value,
        min=get_iot_impact_factor(function_blocks.IMPACT_KEY, function_blocks.hsl_level.value, impact_type) * function_blocks.units.min,
        max=get_iot_impact_factor(function_blocks.IMPACT_KEY, function_blocks.hsl_level.value, impact_type) * function_blocks.units.max
    )

    impact.allocate(duration, function_blocks.usage.hours_life_time)

    return impact.value, impact.min, impact.max, []


def hdd_impact_embedded(impact_type: str, duration: int, hdd: ComponentHDD) -> ComputedImpacts:
    impact = Impact(
        value=get_impact_factor(item='hdd', impact_type=impact_type)['impact'] * hdd.units.value,
        min=get_impact_factor(item='hdd', impact_type=impact_type)['impact'] * hdd.units.min,
        max=get_impact_factor(item='hdd', impact_type=impact_type)['impact'] * hdd.units.max
    )

    impact.allocate(duration, hdd.usage.hours_life_time)

    return impact.value, impact.min, impact.max, ["End of life is not included in the calculation"]


def motherboard_impact_embedded(impact_type: str, duration: int, motherboard: ComponentMotherboard) -> ComputedImpacts:
    impact = Impact(
        value=get_impact_factor(item='motherboard', impact_type=impact_type)['impact'] * motherboard.units.value,
        min=get_impact_factor(item='motherboard', impact_type=impact_type)['impact'] * motherboard.units.min,
        max=get_impact_factor(item='motherboard', impact_type=impact_type)['impact'] * motherboard.units.max
    )

    impact.allocate(duration, motherboard.usage.hours_life_time)

    return impact.value, impact.min, impact.max, ["End of life is not included in the calculation"]


def power_supply_impact_embedded(impact_type: str, duration: int,
                                 power_supply: Component) -> ComputedImpacts:
    if isinstance(power_supply, ComponentFunctionalBlock):
        return iot_functional_blocks_impact_embedded(impact_type, duration, power_supply)
    elif isinstance(power_supply, ComponentPowerSupply):
        return server_power_supply_impact_embedded(impact_type, duration, power_supply)
    else:
        raise NotImplementedError


def server_power_supply_impact_embedded(impact_type: str, duration: int,
                                        power_supply: ComponentPowerSupply) -> ComputedImpacts:
    impact_factor = Impact(
        value=get_impact_factor(item='power_supply', impact_type=impact_type)['impact'],
        min=get_impact_factor(item='power_supply', impact_type=impact_type)['impact'],
        max=get_impact_factor(item='power_supply', impact_type=impact_type)['impact']
    )

    impact = Impact(
        value=power_supply.unit_weight.value * impact_factor.value * power_supply.units.value,
        min=power_supply.unit_weight.min * impact_factor.min * power_supply.units.min,
        max=power_supply.unit_weight.max * impact_factor.max * power_supply.units.max
    )

    impact.allocate(duration, power_supply.usage.hours_life_time)

    return impact.value, impact.min, impact.max, ["End of life is not included in the calculation"]


def ram_impact_embedded(impact_type: str, duration: int, ram: ComponentRAM) -> ComputedImpacts:
    ram_die_impact = Impact(
        value=get_impact_factor(item='ram', impact_type=impact_type)['die_impact'],
        min=get_impact_factor(item='ram', impact_type=impact_type)['die_impact'],
        max=get_impact_factor(item='ram', impact_type=impact_type)['die_impact']
    )

    ram_impact = Impact(
        value=get_impact_factor(item='ram', impact_type=impact_type)['impact'],
        min=get_impact_factor(item='ram', impact_type=impact_type)['impact'],
        max=get_impact_factor(item='ram', impact_type=impact_type)['impact']
    )

    impact = Impact(
        value=((ram.capacity.value / ram.density.value) * ram_die_impact.value + ram_impact.value) * ram.units.value,
        min=((ram.capacity.min / ram.density.max) * ram_die_impact.min + ram_impact.min) * ram.units.min,
        max=((ram.capacity.max / ram.density.min) * ram_die_impact.max + ram_impact.max) * ram.units.max
    )

    impact.allocate(duration, ram.usage.hours_life_time)

    return impact.value, impact.min, impact.max, ["End of life is not included in the calculation"]


def ram_impact_use(impact_type: str, duration: int, ram: ComponentRAM) -> ComputedImpacts:
    impact_factor = ram.usage.elec_factors[impact_type]

    if not ram.usage.avg_power.is_set():
        modeled_consumption = ram.model_power_consumption()
        ram.usage.avg_power.set_completed(
            modeled_consumption.value,
            min=modeled_consumption.min,
            max=modeled_consumption.max,
        )

    impact = ImpactFactor(
        value=impact_factor.value * (
                ram.usage.avg_power.value / 1000) * ram.usage.use_time_ratio.value * duration,
        min=impact_factor.min * (
                ram.usage.avg_power.min / 1000) * ram.usage.use_time_ratio.min * duration,
        max=impact_factor.max * (
                ram.usage.avg_power.max / 1000) * ram.usage.use_time_ratio.max * duration
    )

    return impact.value, impact.min, impact.max, []


def ssd_impact_embedded(impact_type: str, duration: int, ssd: ComponentSSD) -> ComputedImpacts:
    ssd_die_impact = Impact(
        value=get_impact_factor(item='ssd', impact_type=impact_type)['die_impact'],
        min=get_impact_factor(item='ssd', impact_type=impact_type)['die_impact'],
        max=get_impact_factor(item='ssd', impact_type=impact_type)['die_impact']
    )
    ssd_impact = Impact(
        value=get_impact_factor(item='ssd', impact_type=impact_type)['impact'],
        min=get_impact_factor(item='ssd', impact_type=impact_type)['impact'],
        max=get_impact_factor(item='ssd', impact_type=impact_type)['impact']
    )

    impact = Impact(
        value=((ssd.capacity.value / ssd.density.value) * ssd_die_impact.value + ssd_impact.value) * ssd.units.value,
        min=((ssd.capacity.min / ssd.density.max) * ssd_die_impact.min + ssd_impact.min) * ssd.units.min,
        max=((ssd.capacity.max / ssd.density.min) * ssd_die_impact.max + ssd_impact.max) * ssd.units.max
    )

    impact.allocate(duration, ssd.usage.hours_life_time)

    return impact.value, impact.min, impact.max, ["End of life is not included in the calculation"]


def iot_impact_embedded(impact_type: str, duration: int, iot_device: DeviceIoT) -> ComputedImpacts:
    impacts = []
    min_impacts = []
    max_impacts = []
    warnings = iot_device.WARNINGS

    for component in iot_device.components:
        single_impact = compute_single_impact(component, "embedded", impact_type, duration)
        impacts.append(single_impact.value)
        min_impacts.append(single_impact.min)
        max_impacts.append(single_impact.max)
        warnings = warnings + single_impact.warnings

    return sum(impacts) * iot_device.units.value, sum(min_impacts) * iot_device.units.min, sum(max_impacts) * iot_device.units.max, warnings


def iot_impact_use(impact_type: str, duration: int, iot_device: DeviceIoT) -> ComputedImpacts:
    if iot_device.usage.avg_power.value is None:
        raise NotImplementedError

    impact_factor = iot_device.usage.elec_factors[impact_type]
    impact = impact_factor.value * (
            iot_device.usage.avg_power.value / 1000) * iot_device.usage.use_time_ratio.value * duration * iot_device.units.value
    min_impact = impact_factor.min * (
            iot_device.usage.avg_power.min / 1000) * iot_device.usage.use_time_ratio.min * duration * iot_device.units.min
    max_impact = impact_factor.max * (
            iot_device.usage.avg_power.max / 1000) * iot_device.usage.use_time_ratio.max * duration * iot_device.units.max

    return impact, min_impact, max_impact, []


def server_impact_embedded(impact_type: str, duration: int, server: DeviceServer) -> ComputedImpacts:
    impacts = []
    min_impacts = []
    max_impacts = []
    warnings = []

    try:
        for component in server.components:
            component.usage.hours_life_time = server.usage.hours_life_time
            single_impact = compute_single_impact(component, "embedded", impact_type, duration)

            if single_impact is None:
                raise NotImplementedError

            impacts.append(single_impact.value)
            min_impacts.append(single_impact.min)
            max_impacts.append(single_impact.max)
            warnings = warnings + single_impact.warnings

        return sum(impacts), sum(min_impacts), sum(max_impacts), warnings

    except NotImplementedError:
        impact = Impact(value=get_impact_factor(item='SERVER', impact_type=impact_type)["impact"],
                        min=get_impact_factor(item='SERVER', impact_type=impact_type)["impact"],
                        max=get_impact_factor(item='SERVER', impact_type=impact_type)["impact"])

        warnings = ["Generic data used for impact calculation."]

        impact.allocate(duration, server.usage.hours_life_time)

        return impact.value * server.units.value, impact.min * server.units.min, impact.max * server.units.max, warnings


def server_impact_use(impact_type: str, duration: int, server: DeviceServer) -> ComputedImpacts:
    impact_factor = server.usage.elec_factors[impact_type]

    if not server.usage.avg_power.is_set():
        modeled_consumption = server.model_power_consumption()
        server.usage.avg_power.set_completed(
            modeled_consumption.value,
            min=modeled_consumption.min,
            max=modeled_consumption.max
        )

    # Compute impacts at component level
    compute_single_impact(server.cpu, USE, impact_type, duration)
    for ram in server.ram:
        compute_single_impact(ram, USE, impact_type, duration)

    impact = impact_factor.value * (server.usage.avg_power.value / 1000) * server.usage.use_time_ratio.value * duration * server.units.value
    min_impact = impact_factor.min * (server.usage.avg_power.min / 1000) * server.usage.use_time_ratio.min * duration * server.units.min
    max_impact = impact_factor.max * (server.usage.avg_power.max / 1000) * server.usage.use_time_ratio.max * duration * server.units.max

    return impact, min_impact, max_impact, []


def cloud_impact_embedded(impact_type: str, duration: int, cloud_instance: ServiceCloudInstance) -> ComputedImpacts:
    impacts = []
    min_impacts = []
    max_impacts = []
    warnings = []
    default_allocation = cloud_instance.vcpu.value / cloud_instance.platform.get_total_vcpu()

    try:
        for component in cloud_instance.platform.components:
            component.usage.hours_life_time = cloud_instance.platform.usage.hours_life_time
            allocation = default_allocation
            if component.NAME == "RAM":
                allocation = cloud_instance.memory.value / cloud_instance.platform.get_total_memory()
            if component.NAME == "SSD":
                if cloud_instance.ssd_storage.has_value():
                    allocation = cloud_instance.ssd_storage.value / cloud_instance.platform.get_total_disk_capacity("SSD")
                else:
                    continue
            if component.NAME == "HDD":
                if cloud_instance.hdd_storage.has_value():
                    allocation = cloud_instance.hdd_storage.value / cloud_instance.platform.get_total_disk_capacity("HDD")
                else:
                    continue

            single_impact = compute_single_impact(component, "embedded", impact_type, duration, allocation)

            if single_impact is None:
                raise NotImplementedError

            impacts.append(single_impact.value)
            min_impacts.append(single_impact.min)
            max_impacts.append(single_impact.max)
            warnings = warnings + single_impact.warnings
        return sum(impacts), sum(min_impacts), sum(max_impacts), warnings

    except NotImplementedError:
        impact = Impact(value=get_impact_factor(item='SERVER', impact_type=impact_type)["impact"],
                        min=get_impact_factor(item='SERVER', impact_type=impact_type)["impact"],
                        max=get_impact_factor(item='SERVER', impact_type=impact_type)["impact"])

        warnings = ["Generic data used for impact calculation."]

        impact.allocate(duration, cloud_instance.platform.usage.hours_life_time)

        return impact.value * default_allocation, impact.min * default_allocation, impact.max * default_allocation, warnings


def cloud_impact_use(impact_type: str, duration: int, cloud_instance: ServiceCloudInstance) -> ComputedImpacts:
    platform = cloud_instance.platform
    impact_factor = platform.usage.elec_factors[impact_type]

    if not cloud_instance.usage.avg_power.is_set():
        modeled_consumption = cloud_instance.model_power_consumption()
        cloud_instance.usage.avg_power.set_completed(
            value=modeled_consumption.value,
            min=modeled_consumption.min,
            max=modeled_consumption.max
        )

    # Compute impacts at component level
    compute_single_impact(platform.cpu, USE, impact_type, duration)
    for ram in platform.ram:
        compute_single_impact(ram, USE, impact_type, duration)

    impact = impact_factor.value * (cloud_instance.usage.avg_power.value / 1000) * platform.usage.use_time_ratio.value * duration
    min_impact = impact_factor.min * (cloud_instance.usage.avg_power.min / 1000) * platform.usage.use_time_ratio.min * duration
    max_impact = impact_factor.max * (cloud_instance.usage.avg_power.max / 1000) * platform.usage.use_time_ratio.max * duration

    return impact, min_impact, max_impact, []


impacts_functions = {
    "CPU": {
        "use": cpu_impact_use,
        "embedded": cpu_impact_embedded
    },
    "RAM": {
        "use": ram_impact_use,
        "embedded": ram_impact_embedded
    },
    "SSD": {
        "use": simple_impact_use,
        "embedded": ssd_impact_embedded
    },
    "HDD": {
        "use": simple_impact_use,
        "embedded": hdd_impact_embedded
    },
    "POWER_SUPPLY": {
        "use": not_implemented_function,
        "embedded": power_supply_impact_embedded
    },
    "ASSEMBLY": {
        "use": not_implemented_function,
        "embedded": assembly_impact_embedded
    },
    "CASE": {
        "use": simple_impact_use,
        "embedded": casing_impact_embedded
    },
    "MOTHERBOARD": {
        "use": simple_impact_use,
        "embedded": motherboard_impact_embedded
    },
    "SERVER": {
        "use": server_impact_use,
        "embedded": server_impact_embedded
    },
    "IOT_DEVICE": {
        "use": iot_impact_use,
        "embedded": iot_impact_embedded
    },
    "LAPTOP": {
        "use": simple_impact_use,
        "embedded": simple_embedded
    },
    "DESKTOP": {
        "use": simple_impact_use,
        "embedded": simple_embedded
    },
    "TABLET": {
        "use": simple_impact_use,
        "embedded": simple_embedded
    },
    "SMARTPHONE": {
        "use": simple_impact_use,
        "embedded": simple_embedded
    },
    "TELEVISION": {
        "use": simple_impact_use,
        "embedded": simple_embedded
    },
    "SMARTWATCH": {
        "use": simple_impact_use,
        "embedded": ram_impact_embedded
    },
    "BOX": {
        "use": simple_impact_use,
        "embedded": simple_embedded
    },
    "USB_STICK": {
        "use": simple_impact_use,
        "embedded": simple_embedded
    },
    "EXTERNAL_SSD": {
        "use": simple_impact_use,
        "embedded": simple_embedded
    },
    "EXTERNAL_HDD": {
        "use": simple_impact_use,
        "embedded": simple_embedded
    },
    "MONITOR": {
        "use": simple_impact_use,
        "embedded": simple_embedded
    },
    "CLOUD_INSTANCE": {
        "use": cloud_impact_use,
        "embedded": cloud_impact_embedded
    },
    "ACTUATORS": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded
    },
    "CASING": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded
    },
    "CONNECTIVITY": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded
    },
    "MEMORY": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded
    },
    "OTHERS": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded
    },
    "PCB": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded
    },
    "SECURITY": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded
    },
    "PROCESSING": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded
    },
    "SENSING": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded
    },
    "USER_INTERFACE": {
        "use": simple_impact_use,
        "embedded": iot_functional_blocks_impact_embedded
    },
    "VR_CONTROLLER": {
        "use": simple_impact_use, 
        "embedded": simple_embedded
    },
    "VR_HEADSET": {
        "use": simple_impact_use, 
        "embedded": simple_embedded
    },
}



#!/usr/bin/env python3

import pandas as pd

vantage_file = "azure_vms_from_vantage.csv"
benchmark_file_linux = "instances_azure_linux.csv"
benchmark_file_windows = "instances_azure_windows.csv"

def main():
    v_csv_data = pd.read_csv(vantage_file)
    b_csv_data_win = pd.read_csv(benchmark_file_windows)
    b_csv_data_lin = pd.read_csv(benchmark_file_linux)
    found_v_in_b = 0
    found_b_in_v = 0
    items_found_in_v = []
    for d in v_csv_data["Name"]:
        d = d.replace(" ", "_")
        #print("d: {}".format(d))
        res_lin = [ i for i in b_csv_data_lin["Name"] if i == d ]
        if len(res_lin) > 0:
            print("Found {} in Benchmark data for Linux: {}".format(d, res_lin))
            found_v_in_b += 1
        else:
            res_win = [ i for i in b_csv_data_win["Name"] if i == d ]
            if len(res_win) > 0:
                print("Found {} in Benchmark data for Windows: {}".format(d, res_win))
                found_v_in_b += 1
            else:
                print("{} NOT FOUND IN ANY BENCH".format(d))
    for d in b_csv_data_win["Name"]:
        res_v = [ i for i in v_csv_data["Name"] if i.replace(" ", "_") == d ]
        if len(res_v) > 0 and d not in items_found_in_v:
            print("Found {} from bench windows in vantage data: {}".format(d, res_v))
            found_b_in_v += 1
            items_found_in_v += d
    for d in b_csv_data_lin["Name"]:
        res_v = [ i for i in v_csv_data["Name"] if i.replace(" ", "_") == d ]
        if len(res_v) > 0 and d not in items_found_in_v:
            print("Found {} from bench linux in vantage data: {}".format(d, res_v))
            found_b_in_v += 1
            items_found_in_v += d

    print("Found {} items from benchmarks in vantage data. Vantage data contains {}".format(found_b_in_v, len(v_csv_data)))
    print("Found {} items from vantage data in benchmarks. Benchmarks contains {}".format(found_v_in_b, len(b_csv_data_lin)+len(b_csv_data_win)))

if __name__ == "__main__":
    main()



from typing import List

from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component import Component
from boaviztapi.model.impact import Assessable
from boaviztapi.model.usage import ModelUsage


class Device(Assessable):
    NAME = None

    def __init__(self, archetype=None, **kwargs):
        super().__init__(**kwargs)
        self.archetype = archetype
        self.type = None
        self.units = Boattribute(
            default=1,
            min=1,
            max=1
        )
        self._usage = None
        self._impacts = {}

    @property
    def usage(self) -> ModelUsage:
        return self._usage

    @usage.setter
    def usage(self, value: int) -> None:
        self._usage = value

    @property
    def components(self) -> List[Component]:
        return []

    def __iter__(self):
        for attr, value in self.__dict__.items():
            yield attr, value



import os
from typing import List, Tuple

import pandas as pd

from boaviztapi.dto import BaseDTO
from boaviztapi.dto.component import CPU
from boaviztapi.model.component import ComponentCPU
from boaviztapi.model.consumption_profile import CPUConsumptionProfileModel
from boaviztapi.model.component.cpu import attributes_from_cpu_name
from pydantic import ConfigDict


class WorkloadPower(BaseDTO):
    load_percentage: float = None
    power_watt: float = None


class ConsumptionProfile(BaseDTO):
    workload: List[WorkloadPower] = None
    model_config = ConfigDict(arbitrary_types_allowed=True)


class ConsumptionProfileCPU(ConsumptionProfile):
    cpu: CPU = None


def mapper_cp(cp_dto: ConsumptionProfile) -> CPUConsumptionProfileModel:
    cp = CPUConsumptionProfileModel()
    if cp_dto.workload is not None:
        cp.workloads.set_input(cp_dto.workload)
    return cp


def mapper_cp_cpu(cp_dto: ConsumptionProfileCPU) -> Tuple[CPUConsumptionProfileModel, ComponentCPU]:
    cpu = ComponentCPU()
    manufacturer, model_range, family, tdp = None, None, None, None

    if cp_dto.cpu.name is not None:
        fuzzy_match = attributes_from_cpu_name(cp_dto.cpu.name)
        if fuzzy_match is not None:
            manufacturer, family, model_range, tdp = fuzzy_match[1:5]

    if cp_dto.cpu.manufacturer is not None:
        cpu.manufacturer.set_input(cp_dto.cpu.manufacturer)
    elif manufacturer is not None:
        cpu.manufacturer.set_completed(manufacturer)

    if cp_dto.cpu.model_range is not None:
        cpu.model_range.set_input(cp_dto.cpu.model_range)
    elif model_range is not None:
        cpu.model_range.set_completed(model_range)

    if cp_dto.cpu.family is not None:
        cpu.model_range.set_input(cp_dto.cpu.family)
    elif family is not None:
        cpu.family.set_completed(family)

    if cp_dto.cpu.tdp is not None:
        cpu.tdp.set_input(cp_dto.cpu.tdp)
    elif tdp is not None:
        cpu.tdp.set_completed(tdp)

    return mapper_cp(cp_dto), cpu



import os
from pathlib import Path

import yaml
from boaviztapi import data_dir

config_file = os.path.join(data_dir, 'factors.yml')
impact_factors = yaml.load(Path(config_file).read_text(), Loader=yaml.CSafeLoader)


def get_impact_factor(item, impact_type) -> dict:
    if impact_factors.get(item):
        if impact_factors.get(item).get(impact_type):
            return impact_factors.get(item).get(impact_type)
    raise NotImplementedError


def get_electrical_impact_factor(usage_location, impact_type) -> dict:
    if impact_factors["electricity"].get(usage_location):
        if impact_factors["electricity"].get(usage_location).get(impact_type):
            return impact_factors["electricity"].get(usage_location).get(impact_type)
    raise NotImplementedError


def get_electrical_min_max(impact_type, type) -> float:
    if impact_factors["electricity"].get("min-max").get(impact_type):
        if impact_factors["electricity"].get("min-max").get(impact_type).get(type):
            return impact_factors["electricity"].get("min-max").get(impact_type).get(type)
    raise NotImplementedError


def get_available_countries(reverse=False):
    if reverse:
        return {v: k for k, v in impact_factors["electricity"]["available_countries"].items()}
    return impact_factors["electricity"]["available_countries"]


def get_available_iot_functional_block():
    if impact_factors.get("IoT"):
        return impact_factors.get("IoT").keys()


def get_available_iot_hsl():
    response = {}
    for functional_block in get_available_iot_functional_block():
        response[functional_block] = impact_factors.get("IoT").get(functional_block).keys()
    return response


def get_iot_impact_factor(functional_block, hsl, impact_type):
    if impact_factors["IoT"].get(functional_block):
        if impact_factors["IoT"].get(functional_block).get(hsl):
            if impact_factors["IoT"].get(functional_block).get(hsl)["manufacture"].get(impact_type) is not None and impact_factors["IoT"].get(functional_block).get(hsl)["eol"].get(impact_type) is not None:
                return (impact_factors["IoT"].get(functional_block).get(hsl)["manufacture"][impact_type]
                        + impact_factors["IoT"].get(functional_block).get(hsl)["eol"][impact_type])
    raise NotImplementedError


"""
_electricity_emission_factors_df = pd.read_csv(
    os.path.join(data_dir, 'electricity/electricity_impact_factors.csv'))
class ElecFactorProvider:
    def get(self, criteria, location, date):
        pass
    def get_range(self, criteria, location, date1, date2):
        pass
class BoaviztaFactors(ElecFactorProvider):
    def get(self, criteria, usage_location, date=None):
        sub = _electricity_emission_factors_df
        sub = sub[sub['code'] == usage_location]
        return float(sub[f"{criteria}_emission_factor"]), sub[f"{criteria}_emission_source"].iloc[0], 0, ["The impact factor is averaged over the year"]
    def get_range(self, criteria, usage_location, date1=None, date2=None):
        return self.get(criteria,usage_location)

class ElectricityMap(ElecFactorProvider):
    auth_token = "6QGqlsF7ZcdUN6TMB7jX9DMsYKeGHbVl"
    url = "https://api-access.electricitymaps.com/2w97h07rvxvuaa1g"
    now = datetime.now()
    def get(self, criteria, location, date):
        zone = self._location_to_em_zone(location)
        if self.now - timedelta(hours=1) < date < self.now + timedelta(hours=1):
            return self._get_current(zone)
        elif date < self.now:
            return self._get_history(zone, date)
        else:
            return NotImplementedError
    def get_range(self, criteria, zone, date1, date2):
        pass
    def _get_current(self, zone):
        reponse = requests.get(f"{self.url}/carbon-intensity/latest?zone={zone}",
                headers={"X-BLOBR-KEY": self.auth_token}).json()

        return reponse["carbonIntensity"]/1000, f"electricity map response : {reponse}", 0, []
    def _get_history(self, zone, date):
        reponse = requests.get(f"{self.url}/carbon-intensity/history?zone={zone}&datetime={date}",
                               headers={"X-BLOBR-KEY": self.auth_token }).json()

        return reponse
    def _location_to_em_zone(self, location):
        return location
"""



from typing import List, Union

from boaviztapi import config
from boaviztapi.model.component import Component, ComponentCPU, ComponentRAM, ComponentSSD, ComponentHDD, \
    ComponentPowerSupply, ComponentCase, ComponentMotherboard, ComponentAssembly
from boaviztapi.model.device.device import Device
from boaviztapi.model.impact import ImpactFactor
from boaviztapi.model.usage import ModelUsageServer
from boaviztapi.service.archetype import get_server_archetype, get_arch_component


class DeviceServer(Device):
    NAME = "SERVER"

    def __init__(self, archetype=get_server_archetype(config["default_server"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)
        self._cpu = None
        self._ram_list = None
        self._disk_list = []
        self._power_supply = None
        self._case = None
        self._motherboard = None
        self._assembly = None
        self._usage = None

        for attr, val in kwargs.items():
            if val is not None:
                self.__setattr__(attr, val)

    @property
    def cpu(self) -> ComponentCPU:
        if self._cpu is None:
            self._cpu = ComponentCPU(archetype=get_arch_component(self.archetype, "CPU"))
        return self._cpu

    @cpu.setter
    def cpu(self, value: List[ComponentCPU]) -> None:
        self._cpu = value

    @property
    def ram(self) -> List[ComponentRAM]:
        if self._ram_list is None:
            self._ram_list = [ComponentRAM(archetype=get_arch_component(self.archetype, "RAM"))]
        return self._ram_list

    @ram.setter
    def ram(self, value: List[ComponentRAM]) -> None:
        self._ram_list = value

    @property
    def disk(self) -> List[Union[ComponentSSD, ComponentHDD]]:
        if not self._disk_list:
            if get_arch_component(self.archetype, "SSD")["units"] not in [{}, {'default':0}]:
                self._disk_list.append(ComponentSSD(archetype=get_arch_component(self.archetype, "SSD")))
            if get_arch_component(self.archetype, "HDD")["units"] not in [{}, {'default':0}]:
                self._disk_list.append(ComponentHDD(archetype=get_arch_component(self.archetype, "HDD")))

        return self._disk_list

    @disk.setter
    def disk(self, value: List[Union[ComponentSSD, ComponentHDD]]) -> None:
        self._disk_list = value

    @property
    def power_supply(self) -> ComponentPowerSupply:
        if self._power_supply is None:
            self._power_supply = ComponentPowerSupply(archetype=get_arch_component(self.archetype, "POWER_SUPPLY"))
        return self._power_supply

    @power_supply.setter
    def power_supply(self, value: List[ComponentPowerSupply]) -> None:
        self._power_supply = value

    @property
    def case(self) -> ComponentCase:
        if self._case is None:
            self._case = ComponentCase(archetype=get_arch_component(self.archetype, "CASE"))
        return self._case

    @case.setter
    def case(self, value: ComponentCase) -> None:
        self._case = value

    @property
    def case_type(self) -> str:
        return self.case.case_type

    @case_type.setter
    def case_type(self, value: str) -> None:
        self.case.case_type = value

    @property
    def motherboard(self) -> ComponentMotherboard:
        if self._motherboard is None:
            self._motherboard = ComponentMotherboard(archetype=get_arch_component(self.archetype, "MOTHERBOARD"))
        return self._motherboard

    @motherboard.setter
    def motherboard(self, value: ComponentMotherboard) -> None:
        self._motherboard = value

    @property
    def assembly(self) -> ComponentAssembly:
        if self._assembly is None:
            self._assembly = ComponentAssembly(archetype=get_arch_component(self.archetype, "ASSEMBLY"))
        return self._assembly

    @assembly.setter
    def assembly(self, value: ComponentAssembly) -> None:
        self._assembly = value

    @property
    def usage(self) -> ModelUsageServer:
        if self._usage is None:
            self._usage = ModelUsageServer(archetype=get_arch_component(self.archetype, "USAGE"))
        return self._usage

    @usage.setter
    def usage(self, value: ModelUsageServer) -> None:
        self._usage = value

    @property
    def components(self) -> List[Component]:
        return [self.assembly] + [self.cpu] + self.ram + self.disk + [self.power_supply] + [self.case] + [
            self.motherboard]

    def model_power_consumption(self):
        conso_cpu = ImpactFactor(
            value=self.cpu.model_power_consumption().value,
            min=self.cpu.model_power_consumption().min,
            max=self.cpu.model_power_consumption().max,
        )
        self.cpu.usage.avg_power.set_completed(value=conso_cpu.value,
                                               min=conso_cpu.min,
                                               max=conso_cpu.max)

        conso_ram = ImpactFactor(
            value=0,
            min=0,
            max=0
        )
        for ram_unit in self.ram:
            conso_ram.value = conso_ram.value + ram_unit.model_power_consumption().value
            conso_ram.min = conso_ram.min + ram_unit.model_power_consumption().min
            conso_ram.max = conso_ram.max + ram_unit.model_power_consumption().max
            ram_unit.usage.avg_power.set_completed(value=conso_ram.value,
                                                   min=conso_ram.min,
                                                   max=conso_ram.max)

        return ImpactFactor(
            value=(conso_cpu.value + conso_ram.value) * (1 + self.usage.other_consumption_ratio.value),
            min=(conso_cpu.min + conso_ram.min) * (1 + self.usage.other_consumption_ratio.min),
            max=(conso_cpu.max + conso_ram.max) * (1 + self.usage.other_consumption_ratio.max)
        )

    def get_total_memory(self):
        memory = 0
        for ram_strip in self.ram:
            memory += ram_strip.capacity.value * ram_strip.units.value
        return memory

    def get_total_disk_capacity(self, disk_type):
        capacity = 0
        for disk in self.disk:
            if disk.NAME == disk_type or disk_type == "all":
                if disk.units.has_value():
                    capacity += disk.capacity.value * disk.units.value
        return capacity

    def get_total_vcpu(self):
        return self.cpu.threads.value * self.cpu.units.value



from typing import List

from boaviztapi import config
from boaviztapi.model.component.functional_block import ComponentFunctionalBlock, get_functional_block
from boaviztapi.model.device import Device
from boaviztapi.model.usage import ModelUsage
from boaviztapi.service.archetype import get_arch_component, get_iot_device_archetype


class DeviceIoT(Device):
    NAME = "IOT_DEVICE"
    WARNINGS = ["Connected object, not including associated digital services (use of network, datacenter, "
                "virtual machines or other terminals not included)", "Do not include the impact of distribution"]

    def __init__(self, archetype=get_iot_device_archetype(config["default_iot_device"]),
                 functional_block: List[ComponentFunctionalBlock] = None):
        super().__init__(archetype=archetype)

        if functional_block is None:
            functional_block = []

        self._functional_block = functional_block

    @property
    def components(self) -> List[ComponentFunctionalBlock]:
        if not self._functional_block:
            for fb in self.archetype:
                if fb == "USAGE":
                    continue
                if self.archetype[fb]["hsl_level"] != {}:
                    self._functional_block.append(get_functional_block(fb)(
                        archetype=get_arch_component(self.archetype, fb)))
        return self._functional_block

    def add_functional_block(self, functional_block):
        self._functional_block.append(functional_block)

    @property
    def usage(self) -> ModelUsage:
        if self._usage is None:
            self._usage = ModelUsage(archetype=get_arch_component(self.archetype, "USAGE"))
        return self._usage

    @usage.setter
    def usage(self, value):
        self._usage = value


from requests import post
from json import dumps
from csv import DictWriter

BOAVIZTAPI_CONTAINER_URL = "http://0.0.0.0:5000/v1/server/"
RESULT_FILE = "sensitivity_analysis_hosts_with_ssd.csv"

query = {"verbose": "false", "criteria": "gwp"}

dasv4_type2 = {
    "model": {"type": "rack"},
    "configuration": {
        "cpu": {"units": 1, "name": "amd epyc 7763"},
        "ram": [{"units": 48, "capacity": 16}],
        "disk": [
            {
                "units": 0,
                "type": "ssd",
                "capacity": 0,
                "density": 50.6,
            }
        ],
    },
    "usage": {"avg_power": 300, "usage_location": "FRA"},
}

dcsv2_type1 = {
    "model": {"type": "rack"},
    "configuration": {
        "cpu": {"units": 1, "name": "intel xeon e-2288g"},
        "ram": [{"units": 8, "capacity": 8}],
        "disk": [
            {
                "units": 0,
                "type": "ssd",
                "capacity": 0,
                "density": 50.6,
            }
        ],
    },
    "usage": {"avg_power": 300, "usage_location": "FRA"},
}

msv2_type1 = {
    "model": {"type": "rack"},
    "configuration": {
        "cpu": {"units": 8, "name": "intel xeon platinum 8180m"},
        "ram": [{"units": 48, "capacity": 64}],
        "disk": [
            {
                "units": 0,
                "type": "ssd",
                "capacity": 0,
                "density": 50.6,
            }
        ],
    },
    "usage": {"avg_power": 300, "usage_location": "FRA"},
}

hosts = {
    "dcsv2_type1": dcsv2_type1,
    "dasv4_type2": dasv4_type2,
    "msv2_type1": msv2_type1,
}


def get_manufacturing_impact_with_ssd(host, ssd_units, ssd_capacity):

    host["configuration"]["disk"][0]["units"] = ssd_units
    host["configuration"]["disk"][0]["capacity"] = ssd_capacity

    request = post(BOAVIZTAPI_CONTAINER_URL, params=query, data=dumps(host))
    server_data = request.json()
    lifetime_manufacturing_impact = server_data["impacts"]["gwp"]["embedded"]["value"]
    return lifetime_manufacturing_impact


with open(RESULT_FILE, newline="", mode="w") as result_file:
    fieldnames = [
        "id",
        "cpu",
        "impact_manufacturing_no_ssd",
        "impact_manufacturing_ssd_1024_gib",
        "impact_manufacturing_ssd_2048_gib",
        "impact_manufacturing_ssd_4096_gib",
        "impact_manufacturing_ssd_8192_gib",
        "impact_manufacturing_ssd_16384_gib",
    ]
    result_writer = DictWriter(result_file, fieldnames)
    result_writer.writeheader()
    for host in hosts:
        result_writer.writerow(
            {
                "id": host,
                "cpu": hosts[host]["configuration"]["cpu"]["name"],
                "impact_manufacturing_no_ssd": get_manufacturing_impact_with_ssd(
                    hosts[host], 0, 0
                ),
                "impact_manufacturing_ssd_1024_gib": get_manufacturing_impact_with_ssd(
                    hosts[host], 1, 1024
                ),
                "impact_manufacturing_ssd_2048_gib": get_manufacturing_impact_with_ssd(
                    hosts[host], 1, 2048
                ),
                "impact_manufacturing_ssd_4096_gib": get_manufacturing_impact_with_ssd(
                    hosts[host], 1, 4096
                ),
                "impact_manufacturing_ssd_8192_gib": get_manufacturing_impact_with_ssd(
                    hosts[host], 1, 8192
                ),
                "impact_manufacturing_ssd_16384_gib": get_manufacturing_impact_with_ssd(
                    hosts[host], 1, 16384
                ),
            }
        )



from boaviztapi import config
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component.component import Component
from boaviztapi.service.archetype import get_arch_value, get_component_archetype


class ComponentCase(Component):
    AVAILABLE_CASE_TYPE = ['blade', 'rack']
    NAME = "CASE"

    def __init__(self, archetype=get_component_archetype(config["default_case"], "case"), **kwargs):
        super().__init__(archetype=archetype, **kwargs)
        self.case_type = Boattribute(
            default=get_arch_value(archetype, 'case_type', 'default'),
            min=get_arch_value(archetype, 'case_type', 'min'),
            max=get_arch_value(archetype, 'case_type', 'max')
        )


import os

import pandas as pd

import boaviztapi.utils.roundit as rd
from boaviztapi import config, data_dir
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component.component import Component
from boaviztapi.model.consumption_profile import CPUConsumptionProfileModel
from boaviztapi.model.impact import ImpactFactor
from boaviztapi.service.archetype import get_component_archetype, get_arch_value
from boaviztapi.utils.fuzzymatch import fuzzymatch_attr_from_cpu_name, fuzzymatch_attr_from_pdf

_cpu_specs = pd.read_csv(os.path.join(data_dir, 'crowdsourcing/cpu_specs.csv'))


def attributes_from_cpu_name(cpu_name: str):
    return fuzzymatch_attr_from_cpu_name(cpu_name, _cpu_specs)


class ComponentCPU(Component):
    NAME = "CPU"
    name_completion = False

    def __init__(self, archetype=get_component_archetype(config["default_cpu"], "cpu"), **kwargs):
        super().__init__(archetype=archetype, **kwargs)
        self.core_units = Boattribute(
            complete_function=self._complete_from_name,
            default=get_arch_value(archetype, 'core_units', 'default'),
            min=get_arch_value(archetype, 'core_units', 'min'),
            max=get_arch_value(archetype, 'core_units', 'max')
        )
        self.die_size_per_core = Boattribute(
            unit="mm2",
            default=get_arch_value(archetype, 'die_size_per_core', 'default'),
            min=get_arch_value(archetype, 'die_size_per_core', 'min'),
            max=get_arch_value(archetype, 'die_size_per_core', 'max')
        )
        self.die_size = Boattribute(
            complete_function=self._complete_die_size,
            unit="mm2",
            default=get_arch_value(archetype, 'die_size', 'default'),
            min=get_arch_value(archetype, 'die_size', 'min'),
            max=get_arch_value(archetype, 'die_size', 'max')
        )
        self.model_range = Boattribute(
            default=get_arch_value(archetype, 'model_range', 'default'),
            min=get_arch_value(archetype, 'model_range', 'min'),
            max=get_arch_value(archetype, 'model_range', 'max')
        )
        self.manufacturer = Boattribute(
            default=get_arch_value(archetype, 'manufacturer', 'default'),
            min=get_arch_value(archetype, 'manufacturer', 'min'),
            max=get_arch_value(archetype, 'manufacturer', 'max')
        )
        self.family = Boattribute(
            complete_function=self._complete_from_name,
            default=get_arch_value(archetype, 'family', 'default'),
            min=get_arch_value(archetype, 'family', 'min'),
            max=get_arch_value(archetype, 'family', 'max')
        )
        self.name = Boattribute(
            default=get_arch_value(archetype, 'name', 'default'),
            min=get_arch_value(archetype, 'name', 'min'),
            max=get_arch_value(archetype, 'name', 'max')
        )
        self.tdp = Boattribute(
            complete_function=self._complete_from_name,
            unit="W",
            default=get_arch_value(archetype, 'tdp', 'default'),
            min=get_arch_value(archetype, 'tdp', 'min'),
            max=get_arch_value(archetype, 'tdp', 'max')
        )
        self.threads = Boattribute(
            complete_function=self._complete_from_name,
            default=get_arch_value(archetype, 'threads', 'default'),
            min=get_arch_value(archetype, 'threads', 'min'),
            max=get_arch_value(archetype, 'threads', 'max')
        )

    # TODO: compute min & max
    def model_power_consumption(self) -> ImpactFactor:
        self.usage.consumption_profile = CPUConsumptionProfileModel()

        self.usage.consumption_profile.compute_consumption_profile_model(cpu_manufacturer=self.manufacturer.value,
                                                                         cpu_model_range=self.model_range.value,
                                                                         cpu_tdp=self.tdp.value)

        if type(self.usage.time_workload.value) in (float, int):
            self.usage.avg_power.set_completed(
                self.usage.consumption_profile.apply_consumption_profile(self.usage.time_workload.value))
        else:
            self.usage.avg_power.set_completed(
                self.usage.consumption_profile.apply_multiple_workloads(self.usage.time_workload.value))

        return ImpactFactor(
            value=rd.round_to_sigfig(self.usage.avg_power.value, 5) * self.units.value,
            min=rd.round_to_sigfig(self.usage.avg_power.value, 5) * self.units.min,
            max=rd.round_to_sigfig(self.usage.avg_power.value, 5) * self.units.max
        )

    def _complete_die_size(self):
        # Make sure all data have been completed from name
        if self.name.has_value():
            self._complete_from_name()

        # If the die_size_per_core have been set we have nothing to do
        if self.die_size.is_set():
            return None

        # If we have a die_size_per_core and core_units, we can compute the die_size
        elif self.die_size_per_core.is_set() and self.core_units.is_set():
            self.die_size.set_completed(value=float(self.die_size_per_core.value * self.core_units.value),
                                        source="die_size_per_core*core_units")
            return None

        # If we cannot have a family, but we have a default die_size we use it
        elif self.die_size.has_value() and not self.family.has_value():
            self.die_size.set_archetype(self.die_size.default)
            return None

        # If the above completion strategies cannot be applied, we use our cpu specs file
        else:
            self._complete_die_size_from_cpu_specs()

    def _complete_from_name(self):
        if self.name.has_value() and not self.name_completion:
            compute_min_max = False
            if self.name.min != self.name.value or self.name.max != self.name.value:
                compute_min_max = True

            cpu_attributes = attributes_from_cpu_name(self.name.value)
            name, manufacturer, family, model_range, tdp, cores, threads, die_size, die_size_source, source = cpu_attributes if (
                    cpu_attributes is not None) else (None, None, None, None, None, None, None, None, None, None)

            if compute_min_max:
                cpu_attributes_min = attributes_from_cpu_name(self.name.min)
                cpu_attributes_max = attributes_from_cpu_name(self.name.max)
                name_min, manufacturer_min, family_min, model_range_min, tdp_min, cores_min, threads_min, die_size_min, die_size_source_min, source_min = cpu_attributes_min if (
                        cpu_attributes_min is not None) else (None, None, None, None, None, None, None, None, None, None)
                name_max, manufacturer_max, family_max, model_range_max, tdp_max, cores_max, threads_max, die_size_max, die_size_source_max, source_max = cpu_attributes_max if (
                        cpu_attributes_max is not None) else (None, None, None, None, None, None, None, None, None, None)
            else:
                name_min, manufacturer_min, family_min, model_range_min, tdp_min, cores_min, threads_min, die_size_min, die_size_source_min, source_min = name, manufacturer, family, model_range, tdp, cores, threads, die_size, die_size_source, source
                name_max, manufacturer_max, family_max, model_range_max, tdp_max, cores_max, threads_max, die_size_max, die_size_source_max, source_max = name, manufacturer, family, model_range, tdp, cores, threads, die_size, die_size_source, source

            if name is not None:
                self.name.set_completed(name, min=name_min, max=name_max, source="fuzzy match")
            if manufacturer is not None:
                self.manufacturer.set_completed(manufacturer, min=manufacturer_min, max=manufacturer_max,
                                                source=f"Completed from name name based on {source}.")
            if family is not None:
                self.family.set_completed(family, min=family_min, max=family_max,
                                          source=f"Completed from name name based on {source}.")
            if model_range is not None:
                self.model_range.set_completed(model_range, min=model_range_min, max=model_range_max,
                                               source=f"Completed from name name based on {source}.")
            if tdp is not None:
                self.tdp.set_completed(tdp, min=tdp_min, max=tdp_max,
                                       source=f"Completed from name name based on {source}.")
            if cores is not None:
                self.core_units.set_completed(cores, min=cores_min, max=cores_max,
                                              source=f"Completed from name name based on {source}.")
            if threads is not None:
                self.threads.set_completed(threads, min=threads_min, max=threads_max,
                                           source=f"Completed from name name based on {source}.")
            if die_size is not None:
                self.die_size.set_completed(die_size, min=die_size_min, max=die_size_max,
                                            source=f"{die_size_source} : Completed from name name based on {source}.")

    def _complete_die_size_from_cpu_specs(self):
        df = _cpu_specs[_cpu_specs["total_die_size"].notna()]

        # Fuzzymatch on the available code_name
        family = fuzzymatch_attr_from_pdf(self.family.value, "code_name",
                                          _cpu_specs) if self.family.has_value() else None

        # if family not in df we set it to None
        if family not in df["code_name"].values:
            family = None

        if family is not None:
            if family != self.family.value:
                self.family.set_changed(family)
            # Filter the cpu_specs file to get only the rows that match the family
            df = df[(_cpu_specs["code_name"] == self.family.value)]

        # If we don't have a core_units, we take the average of the df
        if self.core_units.is_none():
            self.die_size.set_completed(
                value=rd.round_to_sigfig(df["total_die_size"].mean(), 3),
                min=rd.round_to_sigfig(df["total_die_size"].min(), 3),
                max=rd.round_to_sigfig(df["total_die_size"].max(), 3),
                source=f"Average value for {self.family.value if family else 'all families'}"
            )

        # If we have the good number of cores in the cpu_specs file, we take the value
        elif self.core_units.value in df["cores"].values:
            df_value = df[(df["cores"] == self.core_units.value)]
            df_min = df[(df["cores"] == self.core_units.min)]
            df_max = df[(df["cores"] == self.core_units.max)]
            self.die_size.set_completed(
                value=rd.round_to_sigfig(df_value["total_die_size"].mean(), 3),
                min=rd.round_to_sigfig(df_min["total_die_size"].min(), 3),
                max=rd.round_to_sigfig(df_max["total_die_size"].max(), 3),
                source=f"Average value of {self.family.value if family else 'all families'} with {self.core_units.value} cores"
            )

        # If all rows have the same number of cores but different from the given cores_units
        elif len(df.index) == 1 or (not df.empty and (df["cores"] == df["cores"].iloc[0]).all()):
            self.die_size.set_completed(
                value=rd.round_to_sigfig((df["total_die_size"].iloc[0] * self.core_units.value / df['cores'].iloc[0]),
                                         3),
                min=rd.round_to_sigfig((df["total_die_size"].min() * self.core_units.min / df['cores'].iloc[0]), 3),
                max=rd.round_to_sigfig((df["total_die_size"].max() * self.core_units.max / df['cores'].iloc[0]), 3),
                source=f"Rule of three on {df['name'].iloc[0]}"
            )

        # If none of the above works, we compute a linear regression
        else:
            df = df[~df['cores'].isna()]
            cores = df["cores"].values
            total_die_size = df["total_die_size"].values
            x̄ = cores.mean()
            ȳ = total_die_size.mean()
            b = ((cores - x̄) * (total_die_size - ȳ)).sum() / ((cores - x̄) ** 2).sum()
            a = ȳ - b * x̄

            self.die_size.set_completed(
                value=rd.round_to_sigfig((a + b * self.core_units.value), 3),
                min=rd.round_to_sigfig((a + b * self.core_units.min), 3),
                max=rd.round_to_sigfig((a + b * self.core_units.max), 3),
                source=f"Linear regression on {self.family.value if family else 'all families'}"
            )



from boaviztapi import config
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.device.server import DeviceServer
from boaviztapi.model.impact import Assessable, ImpactFactor
from boaviztapi.model.usage import ModelUsage
from boaviztapi.service.archetype import get_server_archetype, get_cloud_instance_archetype, get_arch_value


class Service(Assessable):
    def __init__(self, archetype=None, **kwargs):
        super().__init__(**kwargs)
        self.units = Boattribute(
            default=1,
            min=1,
            max=1
        )
        self.archetype = archetype
        self._usage = None
        self._impacts = {}

    @property
    def usage(self) -> ModelUsage:
        return self._usage

    @usage.setter
    def usage(self, value: int) -> None:
        self._usage = value


class ServiceCloudInstance(Service):
    NAME = "CLOUD_INSTANCE"

    def __init__(self, archetype=get_cloud_instance_archetype(config["default_cloud_instance"], config["default_cloud_provider"]),
                 **kwargs):
        super().__init__(archetype=archetype, **kwargs)

        self._platform = DeviceServer(archetype=get_server_archetype(archetype_name=get_arch_value(archetype, 'platform', 'default')))
        self.vcpu = Boattribute(
            default=get_arch_value(archetype, 'vcpu', 'default'),
            min=get_arch_value(archetype, 'vcpu', 'min'),
            max=get_arch_value(archetype, 'vcpu', 'max')
        )
        self.memory = Boattribute(
            unit="GB",
            default=get_arch_value(archetype, 'memory', 'default'),
            min=get_arch_value(archetype, 'memory', 'min'),
            max=get_arch_value(archetype, 'memory', 'max')
        )
        self.hdd_storage = Boattribute(
            unit="GB",
            default=get_arch_value(archetype, 'hdd_storage', 'default'),
            min=get_arch_value(archetype, 'hdd_storage', 'min'),
            max=get_arch_value(archetype, 'hdd_storage', 'max')
        )
        self.ssd_storage = Boattribute(
            unit="GB",
            default=get_arch_value(archetype, 'ssd_storage', 'default'),
            min=get_arch_value(archetype, 'ssd_storage', 'min'),
            max=get_arch_value(archetype, 'ssd_storage', 'max')
        )

        for attr, val in kwargs.items():
            if val is not None:
                self.__setattr__(attr, val)

    @property
    def platform(self) -> DeviceServer:
        if self._platform is None:
            self._platform = DeviceServer(archetype=get_server_archetype(archetype_name=self.archetype["platform"]))
        return self._platform

    @platform.setter
    def platform(self, value: DeviceServer) -> None:
        self._platform = value

    def model_power_consumption(self):
        vcpu_allocation = self.vcpu.value / self.platform.get_total_vcpu()
        ram_allocation = self.memory.value / self.platform.get_total_memory()

        total_cpu_consumption = self.platform.cpu.model_power_consumption()
        self.platform.cpu.usage.avg_power.set_completed(value=total_cpu_consumption.value * vcpu_allocation,
                                                        min=total_cpu_consumption.min * vcpu_allocation,
                                                        max=total_cpu_consumption.max * vcpu_allocation)

        total_conso_ram = ImpactFactor(
            value=0,
            min=0,
            max=0
        )

        for ram in self.platform.ram:
            ram_consumption = ram.model_power_consumption()

            ram.usage.avg_power.set_completed(value=ram_consumption.value * ram_allocation,
                                              min=ram_consumption.min * ram_allocation,
                                              max=ram_consumption.max * ram_allocation)

            total_conso_ram.value = total_conso_ram.value + ram.usage.avg_power.value
            total_conso_ram.min = total_conso_ram.min + ram.usage.avg_power.min
            total_conso_ram.max = total_conso_ram.max + ram.usage.avg_power.max

        return ImpactFactor(
            value=(self.platform.cpu.usage.avg_power.value + total_conso_ram.value) * (1 + self.platform.usage.other_consumption_ratio.value),
            min=(self.platform.cpu.usage.avg_power.min + total_conso_ram.min) * (1 + self.platform.usage.other_consumption_ratio.min),
            max=(self.platform.cpu.usage.avg_power.max + total_conso_ram.max) * (1 + self.platform.usage.other_consumption_ratio.max)
        )

    def __iter__(self):
        for attr, value in self.__dict__.items():
            yield attr, value


import os
import sys
from pathlib import Path

import yaml

if "pytest" in sys.modules:
    data_dir = os.path.join(os.path.dirname(__file__), '../tests/data')
else:
    data_dir = os.path.join(os.path.dirname(__file__), 'data')

config_file = os.path.join(data_dir, 'config.yml')
config = yaml.safe_load(Path(config_file).read_text())


import json
import logging

import anyio
import markdown
import toml
from fastapi.middleware.cors import CORSMiddleware
import os
from fastapi import FastAPI
from fastapi.openapi.utils import get_openapi
from mangum import Mangum
from starlette.requests import Request
from starlette.responses import Response
import contextlib
import time
import threading
import uvicorn

from boaviztapi.routers.component_router import component_router
from boaviztapi.routers.consumption_profile_router import consumption_profile
from boaviztapi.routers.iot_router import iot
from boaviztapi.routers.peripheral_router import peripheral_router
from boaviztapi.routers.server_router import server_router
from boaviztapi.routers.cloud_router import cloud_router
from boaviztapi.routers.terminal_router import terminal_router
from boaviztapi.routers.utils_router import utils_router
from boaviztapi.utils.get_version import get_version_from_pyproject

from fastapi.responses import HTMLResponse


# Serverless frameworks adds a 'stage' prefix to the route used to serve applications
# We have to manage it to expose openapi doc on aws and generate proper links.
stage = os.environ.get('STAGE', None)
openapi_prefix = f"/{stage}" if stage else "/"
app = FastAPI(root_path=openapi_prefix)  # Here is the magic
version = get_version_from_pyproject()
_logger = logging.getLogger(__name__)

origins = json.loads(os.getenv("ALLOWED_ORIGINS", '["*"]'))


# Ensure that even an uncaught exception includes CORS headers.
async def catch_exceptions_middleware(request: Request, call_next):
    try:
        return await call_next(request)
    except Exception as e:
        # ignore anyio's EndOfStream exception traceback which just clutters up logs
        if isinstance(e.__context__, anyio.EndOfStream):
            e.__suppress_context__ = True

        _logger.exception(str(e), exc_info=e)
        return Response('Internal Server Error', status_code=500)


app.middleware('http')(catch_exceptions_middleware)

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_methods=["*"],
    allow_headers=["*"],
)

app.include_router(server_router)
app.include_router(cloud_router)
app.include_router(terminal_router)
app.include_router(peripheral_router)
app.include_router(component_router)
app.include_router(iot)
app.include_router(consumption_profile)
app.include_router(utils_router)

if __name__ == '__main__':
    import uvicorn

    uvicorn.run('main:app', host='localhost', port=5000, reload=True)


@app.on_event("startup")
def my_schema():
    intro = open(os.path.join(os.path.dirname(__file__), 'routers/openapi_doc/intro_openapi.md'), 'r', encoding='utf-8')
    openapi_schema = get_openapi(
        title="BOAVIZTAPI - DEMO",
        version=version,
        description=markdown.markdown(intro.read()),
        routes=app.routes,
        servers=app.servers,
    )
    app.openapi_schema = openapi_schema

    return app.openapi_schema


# Wrapper for aws/lambda serverless app
handler = Mangum(app)


@app.get("/", response_class=HTMLResponse)
async def welcome_page():
    html_content = """
    <html>
        <head>
            <title>BOAVIZTAPI</title>
            <style>
                * {
                    font-family: sans-serif;
                }
            </style>
        </head>
        <body>
            <p align="center">
                <img src="https://boavizta.org/media/site/d84925bc94-1642413712/boavizta-logo-4.png" width="100">
            </p>
            <h1 align="center">
              Welcome to BOAVIZTAPI
            </h1>
            <h2 align="center">
              Multicriteria & multistep impacts evaluations for digital assets
            </h2>
            </br>
            <h3 align="center">See our github repository : <a href="https://github.com/Boavizta/boaviztapi">LINK</a></h2>
            <h3 align="center">See OpenAPI specs (swagger) : <a href="docs">LINK</a></h2>
            <h3 align="center">See our complete documentation : <a href="https://doc.api.boavizta.org/">LINK</a></h2>
            <h3 align="center">See the other resources of Boavizta : <a href="https://boavizta.org/">LINK</a> </h2>
            %s
        </body>
    </html>
    """ % os.getenv('SPECIAL_MESSAGE', '')
    return HTMLResponse(content=html_content, status_code=200)


# # A uvicorn server that can be run in a thread. Taken from @florimondmanca @
# https://github.com/encode/uvicorn/issues/742#issuecomment-674411676
class UvicornServerThreaded(uvicorn.Server):
    def install_signal_handlers(self):
        pass

    @contextlib.contextmanager
    def run_in_thread(self):
        thread = threading.Thread(target=self.run)
        thread.start()
        try:
            while not self.started:
                time.sleep(1e-3)
            yield
        finally:
            self.should_exit = True
            thread.join()



import os
from typing import List, Union, Optional

from fastapi import APIRouter, Body, HTTPException, Query

from boaviztapi import config, data_dir
from boaviztapi.dto.device import Server
from boaviztapi.dto.device.device import mapper_server
from boaviztapi.model.device import Device
from boaviztapi.model.device.server import DeviceServer
from boaviztapi.routers.openapi_doc.descriptions import server_impact_by_model_description, \
    server_impact_by_config_description, all_archetype_servers, get_archetype_config_desc
from boaviztapi.routers.openapi_doc.examples import server_configuration_examples_openapi
from boaviztapi.service.archetype import get_server_archetype, get_device_archetype_lst
from boaviztapi.service.verbose import verbose_device
from boaviztapi.service.impacts_computation import compute_impacts

server_router = APIRouter(
    prefix='/v1/server',
    tags=['server']
)


@server_router.get('/archetypes',
                   description=all_archetype_servers)
async def server_get_all_archetype_name():
    return get_device_archetype_lst(os.path.join(data_dir, 'archetypes/server.csv'))


@server_router.get('/archetype_config',
                   description=get_archetype_config_desc)
async def get_archetype_config(archetype: str = Query(example=config["default_server"])):
    result = get_server_archetype(archetype)
    if not result:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")
    return result


@server_router.get('/',
                   description=server_impact_by_model_description)
async def server_impact_from_model(archetype: str = config["default_server"],
                                   verbose: bool = True,
                                   duration: Optional[float] = config["default_duration"],
                                   criteria: List[str] = Query(config["default_criteria"])):
    archetype_config = get_server_archetype(archetype)

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    model_server = DeviceServer(archetype=archetype_config)

    return await server_impact(
        device=model_server,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


@server_router.post('/',
                    description=server_impact_by_config_description)
async def server_impact_from_configuration(
        server: Server = Body(None, openapi_examples=server_configuration_examples_openapi),
        verbose: bool = True,
        duration: Optional[float] = config["default_duration"],
        archetype: str = config["default_server"],
        criteria: List[str] = Query(config["default_criteria"])):
    archetype_config = get_server_archetype(archetype)

    if not archetype_config:
        raise HTTPException(status_code=404, detail=f"{archetype} not found")

    completed_server = mapper_server(server, archetype=archetype_config)

    return await server_impact(
        device=completed_server,
        verbose=verbose,
        duration=duration,
        criteria=criteria
    )


async def server_impact(device: Device,
                        verbose: bool,
                        duration: Optional[float] = config["default_duration"],
                        criteria: List[str] = Query(config["default_criteria"])) -> dict:
    if duration is None:
        duration = device.usage.hours_life_time.value

    impacts = compute_impacts(model=device, selected_criteria=criteria, duration=duration)

    if verbose:
        return {
            "impacts": impacts,
            "verbose": verbose_device(device, selected_criteria=criteria, duration=duration)
        }
    return {"impacts": impacts}



from abc import ABC

from boaviztapi import config
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.device import Device
from boaviztapi.model.usage import ModelUsage
from boaviztapi.service.archetype import get_arch_value, get_user_terminal_archetype, get_arch_component


class EndUserDevice(Device):
    NAME = None

    def __init__(self, archetype=None, **kwargs):
        super().__init__(archetype=archetype, **kwargs)

    @property
    def usage(self) -> ModelUsage:
        if self._usage is None:
            self._usage = ModelUsage(archetype=get_arch_component(self.archetype, "USAGE"))
        return self._usage

    @usage.setter
    def usage(self, value: int) -> None:
        self._usage = value

    def __iter__(self):
        for attr, value in self.__dict__.items():
            yield attr, value


class DeviceLaptop(EndUserDevice, ABC):
    NAME = "LAPTOP"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_laptop"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)
        self.type = Boattribute(
            default=get_arch_value(archetype, 'type', 'default'),
            min=get_arch_value(archetype, 'type', 'min'),
            max=get_arch_value(archetype, 'type', 'max')
        )

class DeviceDesktop(EndUserDevice, ABC):
    NAME = "DESKTOP"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_desktop"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)
        self.type = Boattribute(
            default=get_arch_value(archetype, 'type', 'default'),
            min=get_arch_value(archetype, 'type', 'min'),
            max=get_arch_value(archetype, 'type', 'max')
        )

class DeviceTablet(EndUserDevice, ABC):
    NAME = "TABLET"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_tablet"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)


class DeviceSmartphone(EndUserDevice, ABC):
    NAME = "SMARTPHONE"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_smartphone"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)


class DeviceTelevision(EndUserDevice, ABC):
    NAME = "TELEVISION"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_television"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)
        self.type = Boattribute(
            default=get_arch_value(archetype, 'type', 'default'),
            min=get_arch_value(archetype, 'type', 'min'),
            max=get_arch_value(archetype, 'type', 'max')
        )

class DeviceSmartWatch(EndUserDevice, ABC):
    NAME = "SMARTWATCH"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_smartwatch"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)


class DeviceBox(EndUserDevice, ABC):
    NAME = "BOX"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_box"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)


class DeviceUsbStick(EndUserDevice, ABC):
    NAME = "USB_STICK"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_usb_stick"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)


class DeviceExternalSSD(EndUserDevice, ABC):
    NAME = "EXTERNAL_SSD"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_external_ssd"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)


class DeviceExternalHDD(EndUserDevice, ABC):
    NAME = "EXTERNAL_HDD"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_external_hdd"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)


class DeviceMonitor(EndUserDevice, ABC):
    NAME = "MONITOR"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_monitor"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)

class DeviceVrHeadset(EndUserDevice, ABC):
    NAME = "VR_HEADSET"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_vr_headset"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)
        self.type = Boattribute(
            default=get_arch_value(archetype, 'type', 'default'),
            min=get_arch_value(archetype, 'type', 'min'),
            max=get_arch_value(archetype, 'type', 'max')
        )

class DeviceVrController(EndUserDevice, ABC):
    NAME = "VR_CONTROLLER"

    def __init__(self, archetype=get_user_terminal_archetype(config["default_vr_controller"]), **kwargs):
        super().__init__(archetype=archetype, **kwargs)



from boaviztapi import config
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.component.component import Component
from boaviztapi.service.archetype import get_component_archetype, get_arch_value


class ComponentPowerSupply(Component):
    NAME = "POWER_SUPPLY"

    def __init__(self, archetype=get_component_archetype(config["default_power_supply"], "power_supply"), **kwargs):
        super().__init__(archetype=archetype, **kwargs)

        self.unit_weight = Boattribute(
            unit="kg",
            default=get_arch_value(archetype, 'unit_weight', 'default'),
            min=get_arch_value(archetype, 'unit_weight', 'min'),
            max=get_arch_value(archetype, 'unit_weight', 'max')
        )


from boaviztapi import config
from boaviztapi.model.boattribute import Boattribute
from boaviztapi.model.device import Device
from boaviztapi.model.component import Component
from boaviztapi.model.services.cloud_instance import ServiceCloudInstance, Service


def verbose_cloud(cloud_instance: ServiceCloudInstance, selected_criteria=config["default_criteria"],
                  duration=config["default_duration"]):
    json_output = {**iter_boattribute(cloud_instance),
                   **verbose_usage(cloud_instance),
                   **verbose_device(cloud_instance.platform, selected_criteria=selected_criteria, duration=duration)}
    return json_output


def verbose_device(device: Device, selected_criteria=config["default_criteria"], duration=config["default_duration"]):
    json_output = {"duration": {"value": duration, "unit": "hours"}}
    for component in device.components:
        component.usage.hours_life_time.set_completed(device.usage.hours_life_time.value,
                                                      min=device.usage.hours_life_time.min,
                                                      max=device.usage.hours_life_time.max, source="from device")
        if f"{component.NAME}-1" in json_output:
            i = 2
            while f"{component.NAME}-{i}" in json_output:
                i += 1
            key = f"{component.NAME}-{i}"
        else:
            key = f"{component.NAME}-1"

        json_output[key] = verbose_component(component, selected_criteria, duration)

    json_output = {**json_output, **verbose_usage(device), **iter_boattribute(device)}

    return json_output


def verbose_usage(device: [Device, Component, Service]):
    json_output = {**iter_boattribute(device.usage)}
    if device.usage.consumption_profile is not None:
        if device.usage.consumption_profile.workloads.is_set():
            json_output["workloads"] = device.usage.consumption_profile.workloads.to_json()
            json_output["workloads"]["value"] = [
                {"load_percentage": workload.load_percentage, "power_watt": workload.power_watt} for workload in
                json_output["workloads"]["value"]]
        if device.usage.consumption_profile.params.is_set():
            json_output["params"] = device.usage.consumption_profile.params.to_json()
    for elec in device.usage.elec_factors:
        if device.usage.elec_factors[elec].is_set():
            json_output[f"{elec}_factor"] = device.usage.elec_factors[elec].to_json()

    return json_output


def verbose_component(component: Component, selected_criteria=config["default_criteria"],
                      duration=config["default_duration"]):
    json_output = {"impacts": component.get_impacts(selected_criteria), **iter_boattribute(component),
                   "duration": {"value": duration, "unit": "hours"}}

    if component.usage.avg_power.is_set():
        json_output = {**json_output, **verbose_usage(component)}

    return json_output


def iter_boattribute(element):
    json_output = {}
    for attr, val in element.__iter__():
        if not isinstance(val, Boattribute):
            continue
        if val.is_set():
            json_output[attr] = val.to_json()
    return json_output



from enum import Enum
from typing import Any, Optional


class Status(Enum):
    NONE = "NONE"
    INPUT = "INPUT"
    COMPLETED = "COMPLETED"
    DEFAULT = "DEFAULT"
    CHANGED = "CHANGED"
    ARCHETYPE = "ARCHETYPE"


class Boattribute:
    def __init__(self, **kwargs):

        self._min = None
        self._max = None
        self._value = None
        self.unit = None
        self.status = Status.NONE
        self.source = None
        self.default = None
        self.args = None
        self.warnings = []
        self.complete_function = None

        for attr, val in kwargs.items():
            if val is not None:
                self.__setattr__(attr, val)

    @property
    def value(self) -> Any:
        if self._value is None:
            if self.complete_function:
                self.complete_function()
            if self._value is None and self.default is not None:
                self._value = self.default
                self.status = Status.ARCHETYPE
        return self._value

    @value.setter
    def value(self, value: Any):
        self._value = value

    @property
    def min(self) -> Any:
        if self._min is None:
            self._min = self._value
        return self._min

    @min.setter
    def min(self, value: Any):
        self._min = value

    @property
    def max(self) -> Any:
        if self._max is None:
            self._max = self._value
        return self._max

    @max.setter
    def max(self, value: Any):
        self._max = value

    def add_warning(self, warn):
        self.warnings.append(warn)

    def to_json(self):
        json = {"value": self._value, "status": self.status.value}
        if self.unit: json['unit'] = self.unit
        if self.source: json['source'] = self.source
        if (self._min or self._min==0) and (self.is_default() or self.is_completed() or self.is_archetype()): json['min'] = self._min
        if (self._max or self._max==0) and (self.is_default() or self.is_completed() or self.is_archetype()): json['max'] = self._max
        if self.warnings: json['warnings'] = self.warnings

        return json

    def is_set(self):
        return self.status != Status.NONE

    def is_none(self):
        return self.status == Status.NONE

    def is_input(self):
        return self.status == Status.INPUT

    def is_default(self):
        return self.status == Status.DEFAULT

    def is_completed(self):
        return self.status == Status.COMPLETED

    def is_changed(self):
        return self.status == Status.CHANGED

    def is_archetype(self):
        return self.status == Status.ARCHETYPE

    def set_input(self, value: Any, *, source: Optional[str] = None) -> None:
        self.__set_value_and_status(value, Status.INPUT, source, min=value, max=value)

    def set_completed(self, value: Any, *, source: Optional[str] = None, min = None, max = None) -> None:
        self.__set_value_and_status(value, Status.COMPLETED, source, min=min, max=max)

    def set_default(self, value: Any, *, source: Optional[str] = None) -> None:
        self.__set_value_and_status(value, Status.DEFAULT, source)

    def set_changed(self, value: Any, *, source: Optional[str] = None) -> None:
        self.__set_value_and_status(value, Status.CHANGED, source)

    def set_archetype(self, value: Any, *, source: Optional[str] = None) -> None:
        self.__set_value_and_status(value, Status.ARCHETYPE, source)

    def __set_value_and_status(self, value: Any, status: Status, source: str, min:float = None, max:float=None) -> None:
        self._value = value
        self.status = status
        if source is not None:
            self.source = source
        if min is not None:
            self.min = min
        if max is not None:
            self.max = max

    def has_value(self):
        return (self._value is not None) or (self.default is not None)





#!/usr/bin/python3

import pandas as pd
from pprint import pprint
import re
from math import ceil

target_data = pd.DataFrame({
    "id": [],
    "manufacturer": [],
    "CASE.case_type": [],
    "CPU.units": [],
    "CPU.core_units": [],
    "CPU.die_size_per_core": [],
    #"CPU.tdp": [],
    "CPU.name": [],
    "CPU.vcpu": [],
    "RAM.units": [],
    "RAM.capacity": [],
    "SSD.units": [],
    "SSD.capacity": [],
    "HDD.units": [],
    "HDD.capacity": [],
    "GPU.units": [],
    "GPU.name": [],
    "GPU.memory_capacity": [],
    "POWER_SUPPLY.units": [],
    "POWER_SUPPLY.unit_weight": [],
    "USAGE.time_workload": [],
    "USAGE.use_time_ratio": [],
    "USAGE.hours_life_time": [],
    "USAGE.other_consumption_ratio": [],
    "WARNINGS": []
})

source_columns = ["Dedicated Host SKUs (VM series and Host Type)",
            "Available vCPUs","Available RAM","CPU",
            "Pay as you go"]

source_dedicated_hosts_file = "cleaned_dedicated_hosts.csv"
source_cpu_spec_file = "../../crowdsourcing/cpu_specs.csv"
target_servers_file = "azure_servers.csv"
source_instances_host_mapping = "instance_host.csv"
source_manual_instances_host_mapping = "manual_instance_host_cleaned.csv"
source_vantage_instances_file = "azure_vms_from_vantage.csv"
target_instances_file = "azure_instances.csv"

data = pd.read_csv(source_dedicated_hosts_file)
manual_instances_host_mapping = pd.read_csv(source_manual_instances_host_mapping)
instances_host_mapping = pd.read_csv(source_instances_host_mapping, header=None)
vantage_data = pd.read_csv(source_vantage_instances_file)

def compute_ram_sticks(ram_total_capacity):
    """
    Tries to guess how many RAM sticks are in a host containing
    ram_total_capacity and the associated memory capacity per stick.

    Strategy :
        - keep capacity per stick under 64 GB
        - keep nb_of_sticks below 64 as much as possible, first rule has priority

    returns

    0: nb_of_sticks, the number of RAM sticks
    1: stick_capacity, the capacity per RAM stick
    """
    ram_total_capacity = float(ram_total_capacity.replace(" GiB", "").replace(",", ""))
    stick_capacity = 8.0
    nb_of_sticks = ram_total_capacity / stick_capacity
    while nb_of_sticks > 64:
        if stick_capacity < 64:
            stick_capacity = stick_capacity * 2
        else:
            break;
        nb_of_sticks = ram_total_capacity / stick_capacity
    return nb_of_sticks, stick_capacity

def get_cpu_spec(cpu_specs, cpu_ref_string):
    cpu_ref_string = re.sub(r"(\w+) Generation ", '', cpu_ref_string)
    cpu_ref_string = re.sub(r"([0-9]+\.[0-9]+) (GHz|Ghz) ", '', cpu_ref_string)
    cpu_ref_string = re.sub(r" (\(\w+\)).*", '', cpu_ref_string)
    cpu_ref_string = cpu_ref_string.replace("Scalable Processor", "Platinum")
    #exp = r"(AMD|Intel|Ampere)( Xeon|EPYC|Epyc)( Platinum|Gold|Silver|Bronze|Scalable)( [0-9]+\w*(-)?\w*)"
    #print("parsing {}".format(cpu_ref_string))
    #res = re.finditer(exp, cpu_ref_string)
    spec = None
    #if res is not None:
    #    for match in res:
    #        print("parsing {} gives: {}".format(cpu_ref_string, match))
    for s in cpu_specs[["name", "code_name", "generation", "manufacturer", "foundry", "manufacturer", "release_date", "frequency", "tdp", "cores", "threads", "process_size", "die_size", "io_die_size", "io_process_size", "total_die_size", "model_range"]].iterrows():
        if cpu_ref_string in s[1]["name"] or s[1]["name"] in cpu_ref_string or s[1]["name"].replace(" Xeon", "") in cpu_ref_string.replace("v", "V"):
            spec = s[1]
    return spec

def get_gpu_from_string(cpu_gpu_string):
    exp = re.compile("(AMD|NVIDIA).?.?.? (Radeon|Tesla) (\w+)?( (\w+)?(-)?(\w)*)? (GPUs)?")
    res = exp.search(cpu_gpu_string)
    if res is not None:
        return res.group().replace("GPUs", "")
    else:
        return None

def get_cpu_units(host_data, cpu_data):
    if cpu_data["threads"] != "nan":
        guessed_value = round(host_data["Available vCPUs"] / cpu_data["threads"])
        # sometimes the ratio is weird, sanity check required
        if guessed_value == 0:
            guessed_value = 1
        elif guessed_value == 7:
            guessed_value = 8
        elif guessed_value == 3:
            guessed_value = 4
        return guessed_value
    else:
        return None

def get_instances_per_host(instances_host_mapping, host_name):
    manual_data = manual_instances_host_mapping.copy()
    manual_data.columns = range(manual_data.shape[1])
    instances_host_mapping = pd.concat([
        instances_host_mapping, manual_data[[0, 1]]
    ])
    # TODO: mapping instance -> hostS, when multiple instance refs with different hosts
    # build a min;avg;max mapping, put it in host cell for this instance in the resulting dataframe
    # gen both result.csv in the old fashion way and new result.csv with the new mapping, compare differences in per-instance impact
    mapping = instances_host_mapping[instances_host_mapping[1] == host_name.lower()][0]
    instances_data = pd.DataFrame({
        "id": [],
        "vcpu": [],
        "memory": [],
        "ssd_storage": [],
        "hdd_storage": [],
        "gpu_units": [],
        "platform": [],
        "source": []
    })
    for instance in mapping:
        instance = instance.replace(" ", "_")
        from_vantage = vantage_data[vantage_data["Name"].str.lower() == instance.replace("_", " ")]
        from_vantage_by_api_name = vantage_data[vantage_data["API Name"].str.lower() == instance.replace("_", " ")]

        vcpus = from_vantage["vCPUs"].str.replace(" vCPUs", "")
        if len(vcpus.index) == 0:
            vcpus = from_vantage_by_api_name["vCPUs"].str.replace(" vCPUs", "")
        mem = from_vantage["Instance Memory"].str.replace(" GiB", "")
        if len(mem.index) == 0:
            mem = from_vantage_by_api_name["Instance Memory"].str.replace(" GiB", "")
        gpus = from_vantage["GPUs"].str.replace(r"", "", regex=True)
        if len(gpus.index) == 0:
            gpus = from_vantage_by_api_name["GPUs"].str.replace(r"", "", regex=True)
        ssd_sto = from_vantage["Instance Storage"]
        if len(ssd_sto.index) == 0:
            ssd_sto = from_vantage_by_api_name["Instance Storage"]
        if len(ssd_sto.index) > 0:
            ssd_sto_value = ssd_sto.values[0]
        else:
            ssd_sto_value = 0
        # if we also have NVMe local ssd storage for the instance, add it to ssd_sto
        data_from_manual_mapping = manual_instances_host_mapping[manual_instances_host_mapping["instance"] == instance.replace("standard_", "")]
        if len(data_from_manual_mapping.index) > 0:
            if data_from_manual_mapping["nvme_units"].values[0] != "NaN" and data_from_manual_mapping["nvme_units"].values[0] > 0:
                if data_from_manual_mapping["nvme_capacity"].values[0] != "NaN" and data_from_manual_mapping["nvme_capacity"].values[0] > 0:
                    ssd_sto_value = float(ssd_sto_value) + float(data_from_manual_mapping["nvme_units"].values[0]) * float(data_from_manual_mapping["nvme_capacity"].values[0])
        else:
            print("Couldn't find {} in manual_instance_mapping".format(instance.replace("standard_", "")))
        # if we have so bad data that we don't know at least for vcpus and memory, discard the instance
        if len(vcpus.index) > 0 or len(mem.index) > 0:
            instances_data = pd.concat(
                [
                    instances_data,
                    pd.DataFrame({
                        "id": [instance],
                        "vcpu": [vcpus.values[0] if len(vcpus.index) > 0 else 0],
                        "memory": [mem.values[0] if len(mem.index) > 0 else 0],
                        "ssd_storage": [ssd_sto_value],
                        # TODO: change default SSD value for average of all default storage
                        "hdd_storage": [0],
                        "gpu_units": [gpus.values[0] if len(gpus.index) > 0 else 0],
                        # TODO fetch GPU units per instance
                        "platform": [host_name],
                        "source": ["Vantage for instance specs (https://instances.vantage.sh/azure/), Azure docs for instance-host mapping, see README for details."]
                    })
                ]
            )
    return instances_data

def get_instance_gpus_from_vantage(instance_name):
    res = vantage_data.loc[vantage_data["Name"].str.lower() == instance_name]
    return res

def get_disks_per_platform(platform_data, instances_per_host):
    res = {
        "ssd": {
            "units": 0,
            "capacity": 0
        },
        "hdd": {
            "units": 0,
            "capacity": 0
        }
    }
    ssd_sto_platform = 0
    hdd_sto_platform = 0
    # if we have the max number of instance per host in the manual mapping data
    manual_data = manual_instances_host_mapping[manual_instances_host_mapping["host"] == platform_data["id"].lower()]
    if len(manual_data.index) > 0:
        for i in manual_data[["instance", "host", "nbinstancesmax"]].iterrows():
            if float(i[1]["nbinstancesmax"]) > 0.0:
                instance_data = instances_per_host[instances_per_host["id"] == i[1]["instance"]]
                if len(instance_data["ssd_storage"].index) > 0 and instance_data["ssd_storage"].values[0] * i[1]["nbinstancesmax"] > ssd_sto_platform:
                    ssd_sto_platform = float(instance_data["ssd_storage"].values[0]) * float(i[1]["nbinstancesmax"])
    else: 
    #  else get instance with highest vcpus count
        max_vcpu_instances = instances_per_host[instances_per_host["vcpu"]==instances_per_host["vcpu"].max()]
        if len(max_vcpu_instances.index) > 0:
            max_vcpus_per_instance = max_vcpu_instances["vcpu"].values[0]
            # divide the maximum vcpus count of the host by this max instance vcpus
            max_vcpus_on_host = int(platform_data["vcpus"])
            ratio = max_vcpus_on_host / float(max_vcpus_per_instance)
            # to get how many vms of this type can run on the host : nb_{i}_on_{h}
            # get ssd and hdd units and capacity for this instance
            # multiply those values by nb_{i}_on_{h}, apply to res, return
            ssd_sto_platform = ratio * max_vcpu_instances["ssd_storage"].values[0]
            hdd_sto_platform = ratio * max_vcpu_instances["hdd_storage"].values[0]
            # As we don't have better information so far regarding underlying storage infrastructure, we use an
            # empirical hypothesis that SSD drives are most probably bigger than 2TB and HDD drives bigger than 4TB
    # whatever the source of information we consider at least 2TB capacity per drive
    res["ssd"]["units"] = ceil(ssd_sto_platform / 2048.0)
    if res["ssd"]["units"] > 0:
        res["ssd"]["capacity"] = 2048
    else:
        res["ssd"]["capacity"] = 0
    res["hdd"]["units"] = ceil(hdd_sto_platform / 4096.0)
    if res["hdd"]["units"] > 0:
        res["hdd"]["capacity"] = 4096
    else:
        res["hdd"]["capacity"] = 0
    return res

cpu_specs = pd.read_csv(source_cpu_spec_file)
hosts_matching_no_instance = []
instance_host_mapping = pd.DataFrame(
    {
        "id": [],
        "vcpu": [],
        "memory": [],
        "ssd_storage": [],
        "hdd_storage": [],
        "gpu_units": [],
        "platform": [],
        "source": []
    }
)

for host in data[["Dedicated Host SKUs (VM series and Host Type)", "Available vCPUs","Available RAM","CPU"]].iterrows():
    #
    # Get CPU specification for the current host
    #
    current_cpu_spec = get_cpu_spec(cpu_specs, host[1]["CPU"])
    current_host_name = host[1]["Dedicated Host SKUs (VM series and Host Type)"]
    if current_cpu_spec is None:
        print("No spec found for: {}".format(host[1]["CPU"]))
    else:
        print("CPU: {} threads: {} host max vCPU : {} supposed CPU units: {}".format(current_cpu_spec["name"], current_cpu_spec["threads"], host[1]["Available vCPUs"], get_cpu_units(host[1], current_cpu_spec)))
    current_gpu = get_gpu_from_string(host[1]["CPU"])
    instances_per_host = get_instances_per_host(instances_host_mapping, current_host_name)
    if instances_per_host is None or len(instances_per_host.index) == 0:
        print("host {} not macthing any instance !!!".format(current_host_name))
        hosts_matching_no_instance.append(current_host_name)
    else:
        #
        # Make a mapping instance to host
        #
        instance_host_mapping = pd.concat([instance_host_mapping, instances_per_host])
    #
    # If there are GPUs on this host
    # Get how many GPUs are allocated to virtual machines deployed on current host, as a maximum
    # TODO FIX
    #
    #if current_gpu is not None:
    #    print("Current GPU: {}".format(current_gpu))
    #    for i in instances_per_host:
    #        instance_gpus = get_instance_gpus_from_vantage(i["instance_name"])
    #        if len(instance_gpus.index) > 0:
    #            print("Instance: {} GPU: {}".format(i["instance_name"], instance_gpus))
    #        else:
    #            print("Host {} has GPUs ({}) but instance {} has not".format(current_host_name, current_gpu, i["instance_name"]))

    nb_of_sticks, stick_capacity = compute_ram_sticks(host[1]["Available RAM"])
    platform_id = host[1]["Dedicated Host SKUs (VM series and Host Type)"]
    cpu_units = get_cpu_units(host[1], current_cpu_spec)
    disks_hypothesis = get_disks_per_platform(
        {"id": platform_id, "vcpus": host[1]["Available vCPUs"]},
        instances_per_host
    )
    new_data = pd.DataFrame({
        "id": [platform_id],
        "manufacturer": ["Azure"],
        "CASE.case_type": ["rack"], # TODO: source from Azure docs which platform is blade, which is rack
        "CPU.units": [cpu_units],
        "CPU.core_units": [""],
        "CPU.die_size_per_core": [""],
        #"CPU.tdp": [current_cpu_spec["tdp"] if current_cpu_spec is not None else ""],
        "CPU.name": [current_cpu_spec["name"] if current_cpu_spec is not None else ""],
        "CPU.vcpu": [host[1]["Available vCPUs"]],
        "RAM.units": [nb_of_sticks],
        "RAM.capacity": [stick_capacity],
        "SSD.units": [disks_hypothesis["ssd"]["units"]], # TODO: get hypothesis that seem close to what users see on those machines
        "SSD.capacity": [disks_hypothesis["ssd"]["capacity"]],
        "HDD.units": [disks_hypothesis["hdd"]["units"]],
        "HDD.capacity": [disks_hypothesis["hdd"]["capacity"]],
        "GPU.units": [0], # TODO: guess it from how many GPUs have the biggest instances hosted on this platform
        "GPU.name": [current_gpu],
        "GPU.memory_capacity": [""],
        "POWER_SUPPLY.units": ["2;2;2"], # TODO: source from Azure docs which platform is blade, which is rack
        "POWER_SUPPLY.unit_weight": ["2.99;1;5"], # TODO: source from Azure docs which platform is blade, which is rack
        "USAGE.time_workload": ["50;0;100"],
        "USAGE.use_time_ratio": ["1"],
        "USAGE.hours_life_time": [52560], # According to latest news, Azure extended servers lifetime from 4 to 6 years. (https://www.networkworld.com/article/971373/microsoft-extends-azure-server-lifetimes-by-50.html)
        "USAGE.other_consumption_ratio": ["0.33;0.2;0.6"], # TODO: challenge those factors based on azure actual hardware
        "WARNINGS": ["RAM units and per unit capacity not verified. RAM capacity from Azure docs was: {}".format(host[1]["Available RAM"])]
    })
    target_data = pd.concat([target_data, new_data])

print("Hosts matching no instance: ")
pprint(hosts_matching_no_instance)
instance_host_mapping.to_csv(target_instances_file, index=False)

target_data.to_csv(target_servers_file, index=False)
